//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

function dateFormat(date) 
{
    var datetime = date.getUTCFullYear() + "-";
    var month = String(date.getUTCMonth() + 1);
    datetime += (month.length == 1 ?  '0' + month : month) + "-";
    var day = date.getUTCDate();
    datetime += (day < 10 ? '0' + day : day);

    datetime += 'T';

    var hour = date.getUTCHours();
    datetime += (hour < 10 ? '0' + hour : hour) + ':';
    var minutes = date.getUTCMinutes();
    datetime += (minutes < 10 ? '0' + minutes : minutes) + ':';
    var seconds = date.getUTCSeconds();
    datetime += (seconds < 10 ? '0' + seconds : seconds) + "Z";

    return datetime;
}

function JSAtomAPI() 
{
}

JSAtomAPI.prototype=
{
    _req: null,
    _url: null,

    ERROR_HTTP:   10,
    ERROR_FAULT:   11,
    ERROR_PARSER:   12,

    init: function(url) {
        this._url = url;
    },

    doRequest: function(listener, method, url, body, processor) {
        var inst = this;
        this._req = new XMLHttpRequest();
        this._req.onreadystatechange = function (aEvt) {
            if(inst._req.readyState == 4) {
                dump("\nRESPONSE\n" + inst._req.responseText);
                if(inst._req.status == 200)
                {
                    try
                    {
                        processor(listener, inst);
                    }
                    catch(e)
                    {
                        listener.onError(inst.ERROR_PARSER);
                    }
                }
                else {
                    var fault =  {
                        faultString: "Make sure that are not trying to blog really weird html, and note the following kind user:\n\n",
                    };
                    fault.faultString += inst._req.responseText;
                    listener.onFault(fault);
                }
            }
        };
        //var rval = "";
        rval = this._req.open(method, url, true);
        //dump("Sending: " + url + ">>>" + body);
        //alert(rval);
        rval = this._req.send(body); 
        //alert(rval);
    },

    getRecentPosts: function(listener) {
        this.doRequest(listener,"GET", this._url,null, this.parseRecentPosts);
    },
    getUsersBlogs: function(listener, url) {
        var url = this._url.match(/(.+)\//);
        url = RegExp.$1;
        this.doRequest(listener,"GET", url, null, this.parseUsersBlogs);
    },

    parseUsersBlogs: function(listener, inst) {
        try {
            var rval = new Array();
            var dom = inst._req.responseXML;
            var links = dom.getElementsByTagName("link");
            for(var i=0;i<links.length;++i)
            {
                var link = links[i];
                var title = link.getAttribute("title");
                var rel = link.getAttribute("rel");
                var href = link.getAttribute("href");
                if(rel!="service.post") continue;

                href.match(/.+\/(.+)/);
                var bid = RegExp.$1;

                var obj  =
                {
                    blogName: title,
                    blogid:  bid,
                    url:  href,
                };
                rval[rval.length++] = obj;
            }
            listener.onResult(rval);
        }
        catch(e) {
            //alert(e + " " + e.lineNumber);
        }
    },

    parseRecentPosts: function(listener, inst) {
        var rval = new Array();
        var dom = inst._req.responseXML;
	    var entries = dom.getElementsByTagName("entry");
        for(var i=0;i<entries.length;++i)
        {
            try
            {
                var entry_n = entries[i];
                var link_n = inst.getNamedChild(entry_n, "link");
                var content_n = inst.getNamedChild(entry_n, "content");
                //content_n = inst.getNamedChild(content_n, "div");
                var title_n = inst.getNamedChild(entry_n, "title");
                var created_n = inst.getNamedChild(entry_n, "issued");
                var author_n = inst.getNamedChild(entry_n, "author");
                var name_n = inst.getNamedChild(author_n, "name");

                var href = link_n.getAttribute("href");
                href.match(/.+\/(.+)/);
                var postid = RegExp.$1;

                var dateval = created_n.firstChild.nodeValue;
                //2004-04-12T06:07:20Z
                dateval.match(/(\d+?)-(\d+?)-(\d+?)T(\d+?):(\d+?):(\d+?)-/);
                var dateutc =  Date.UTC(RegExp.$1, RegExp.$2 - 1, RegExp.$3, RegExp.$4, RegExp.$5, RegExp.$6);
                var date = new Date(dateutc);
                //alert(dateval +  " " +  dateFormat(date));

                //alert(content_n.firstChild.nodeValue);
                var xs = new XMLSerializer();
                var obj  =
                {
                    description: inst.getInside(xs.serializeToString(content_n)),
                    title: title_n.firstChild.nodeValue,
                    dateCreated: date,
                    userid:  name_n.firstChild.nodeValue,
                    postid:  postid,
                    
                };
                rval[rval.length] = obj;
            }
            catch(e)
            {
                alert(e);
            }
            listener.onResult(rval);
        }
    }, getInside: function(xhtml)
    {
        xhtml = xhtml.replace(/\r/g, "");
        xhtml = xhtml.replace(/\n/g, "");
        xhtml.match(/.+?<div.+?>(.+)<\/div/);
        return RegExp.$1;
    },
    getNamedChild: function(node, name) {
        for(var i=0;i<node.childNodes.length;++i)
        {
            if(node.childNodes[i].nodeName==name) return node.childNodes[i];
        }
        return null;
    },
    deletePost: function (listener, postid) {
        var url = this._url;
        if(postid) url += "/" + postid;
        this.doRequest(listener,"DELETE", url,null, this.handleDelete);
    },
    doPost: function (listener, method, postid, title, description, date_created, public) {
        var url = this._url;
        if(postid) url += "/" + postid;
        var date = dateFormat(date_created);
        var body = "";
        var version = "Flock Alpha, Version .1";
        body += '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>';
        body += '<entry xmlns="http://purl.org/atom/ns#">';

        var draft = "false";
        if(!public) draft = "true";

        body += '<draft xmlns="http://purl.org/atom-blog/ns#">' + draft + '</draft> '
        body += '<title mode="escaped" type="text/plain">' + title + '</title>';
        body += '<issued>' + date + '</issued>';
        body += '<generator url="http://flock.com">' + version + '</generator>'
        body += '<content type="application/xhtml+xml">'
        description = description.replace(/(&\S+)/g, "");
        //alert(description);
        body += '<div xmlns="http://www.w3.org/1999/xhtml">' + description + '</div>';
        body += '</content> </entry>';
        dump(body);
        this.doRequest(listener,method, url,body, this.parsePosts,public);
    },
    editPost: function (listener, postid, title, description, date_created, public) {
        this.doPost(listener, "PUT", postid, title, description, date_created,public);
    },
    newPost: function (listener, title, description, date_created,public) {
        this.doPost(listener, "POST", null, title, description, date_created,public);
    },
    parsePosts: function(listener, inst) {
        var dom = inst._req.responseXML;
        if(!dom)
        {
            var fault =  {
                faultString: "There was an error posting to your blog.  Make sure that are not trying to blog really weird html.",
            };
            listener.onFault(fault);
            return;
        }
	    var entries = dom.getElementsByTagName("entry");
        var rval = null;
        try
        {
            var entry_n = entries[0];
            var link_n = inst.getNamedChild(entry_n, "link");
            var href = link_n.getAttribute("href");
            href.match(/.+\/(.+)/);
            rval = RegExp.$1;
        }
        catch(e)
        {
            alert(e);
        }
        listener.onResult(rval);
    },
    handleDelete: function(listener, inst) {
        listener.onResult("hack");
    },
}
