//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//


///////////////////////////////////////////////////////////////// 
const R2_BLOG_NS = "http://www.flock.com/rdf#";
const BLOG_STATE_NEW = "NEW";
const BLOG_STATE_CHANGED = "CHANGED";
const BLOG_STATE_OK = "OK";
const BLOG_URN_ROOT = "urn:r2:blog:settings:";
const BLOG_SETTINGS = "urn:r2:blog:settings";
const BLOG_TMP_POST = "urn:r2:blog:tmp:posts";
const BLOG_ALLPOSTS_FOLDER = "urn:r2:blog:AllPosts";
const BLOG_DRAFTS_FOLDER = "urn:r2:blog:settings:Drafts:";
const BLOG_DRAFTS_FOLDER2 = "urn:r2:blog:settings:Drafts";
const BLOG_DRAFTS_NAME = "Drafts";
const BLOG_PREF_ROOT = "urn:r2:blog:preferences:notifications"
const BLOG_RDF_FILE = "blog_alpha.rdf";
const BLOG_POST_DOWNLOAD_COUNT = 10;

var gBlogRDF = null;
var gRefreshList = null;

function flock_error(e) {
    alert(e + " " + e.lineNumber + " " + e.fileName);
}

function flock_blogSetDefaultAccountIndex(i) {
    flock_setIntPref("blog.defaultAccountIndex", i);
}

function flock_blogGetDefaultAccountIndex(i) {
    try {
        return flock_getIntPref("blog.defaultAccountIndex");
    }
    catch(e) {
        return 0;
    }
}

function flock_blogSetDefaultAccount(acct) {
    flock_setPref("blog.defaultAccount", acct);
}

function flock_blogGetDefaultAccount() {
    var defaultAccount = flock_getPref("blog.defaultAccount");
    if(!defaultAccount) flock_setPref("blog.defaultAccount", BLOG_DRAFTS_FOLDER2);
    return flock_getPref("blog.defaultAccount");
}

function flock_blogGetAccountIndex(aAccount) {
    var ds = flock_blogGetDS();
    var settings = ds.getNode("urn:r2:blog:settings");
    var accounts = settings.getChildren();
    var rval = 0;
    while(accounts.hasMoreElements()) {
        var account = accounts.getNext();
        var a = BLOG_URN_ROOT + account.getTarget(R2_BLOG_NS+"title").getValue();

        if(a==aAccount) {
            return rval;
        }
        ++rval;
    }
    return 0;
}

function flock_blogBuildBlogThisPostURI(postid) {
    var url = "chrome://browser/content/flock/blog/blogEditor.xul"

    if(index==null) index = 0;

    var index = 0;
    var selectedBlog = flock_blogGetDefaultAccount();

    var params = "?bloguri="; 
    params += hexEncode(selectedBlog) + "&selectedIndex=" + hexEncode(index);
    params += "&postid=" + hexEncode(postid);
    url += params;
    return url;
}

/////////////////////////////////////////////////////////////////
// A useful, shared command, ... or is it?
/////////////////////////////////////////////////////////////////

var flock_blogNewPostCommand =
{
    isEnabled: function() {
        return true;
    },
    exec: function() {
        var date = new Date();
        var postid = BLOG_TMP_POST;
        var url = flock_blogBuildBlogThisPostURI(postid);
        flock_blogSavePost(postid, null, "<p></p>", BLOG_STATE_NEW, new Date());
        flock_blogDoOpenPost(url);
        return;
    },
}


var flock_blogBlogThisCommand =
{
    isEnabled: function() {
        return true;
    },
    exec: function() {
        var date = new Date();
        //var postid = BLOG_DRAFTS_FOLDER + "posts:" + date.getTime();//Give each post a UID
        var postid = BLOG_TMP_POST;
        var url = flock_blogBuildBlogThisPostURI(postid);

        var selection = $$("content").mCurrentBrowser.contentWindow.getSelection() + "";
        var data = "";
        var location = $$("content").mCurrentBrowser.contentWindow.location + "";
        if(location.match(/^chrome:\/\/.+/))
        {
            location = "";
        }
        var title = $$("content").mCurrentBrowser.contentTitle + "";

        if(selection.length == 0)
        {
            $$("content").mCurrentBrowser.contentWindow.location.spec;
            data = '<a href="' + location + '">' + title + "</a> <br/><br/>";
        }
        else
        {
            data = '<blockquote cite="' + location + '">' + selection + "</blockquote>";
            if(title.length==0) title = location;
            data += '<p class="citation"><cite cite="' + location + '"><a href="' + location + '">' + title + '</a>' + "</cite></p><p></p>";
        }
        data = data.replace(/\n/g,"");
        data = data.replace(/\r/g,"");

        flock_blogSavePost(postid, null, data, BLOG_STATE_NEW, new Date());
        flock_blogDoOpenPost(url);
        return;
    },
}

var flock_blogCreateAccountCommand =
{
    isEnabled: function() {
        return true;
    },
    exec: function() {
        flock_blogOpenWizard();
    },
}

/////////////////////////////////////////////////////////////////
// Dialogs
/////////////////////////////////////////////////////////////////

function flock_blogDoOpenPost(url) {
    try
    {
        if(flock_getBoolPref("blog.editInTab"))
        {
            var t = $$("content").addChromeTab(url, "Blog Editor");
            $$("content").selectedTab = t;
        }
        else
        {
            window.openDialog(url,"_blank", "chrome,close,titlebar,resizable=yes,toolbar,dialog=no,scrollbars=yes");
        }
    }
    catch(e)
    {
        window.openDialog(url);
    }
}

function flock_blogOpenEditor() {
    var t = $$("content").addChromeTab("chrome://browser/content/flock/blog/blogEdit.xul", "Blog Manager");
    $$("content").selectedTab = t;
}

function flock_blogGotoService() {
    flock_goto("http://www.flock.com/fiveways/togetstarted/with/blogging/");
}

function flock_blogOpenWizard(param) {
    var ugly = false;
    if(!param) ugly = true;
    if(!param) param = "";
    if(ugly) window.open("chrome://browser/content/flock/blog/blogAccountWizard.xul", "accountWizard", "chrome,modal,centerscreen", param); 
    else window.openDialog("chrome://browser/content/flock/blog/blogAccountWizard.xul", "accountWizard", "modal,centerscreen", param); 
}


/////////////////////////////////////////////////////////////////
// Initialization of Globals
/////////////////////////////////////////////////////////////////

function flock_blogInit() {
    gDS = flock_blogGetDS();
    gSelectedBlog = null;
    gSelectedPost = null;
    flock_blogInitRDF();
}

function flock_blogGetDS() {
    R2_BLOG_NS = "http://www.flock.com/rdf#";
    if(gBlogRDF) return gBlogRDF;

    var fname = BLOG_RDF_FILE;

    var blogFile = Components.classes["@mozilla.org/file/directory_service;1"].
        getService(Components.interfaces.nsIProperties).
        get("ProfD", Components.interfaces.nsIFile);

    blogFile.append(fname);
    if(!blogFile.exists())
    {
      blogFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
    }

    var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
                              
    var fileHandler = ioSvc.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var furl = fileHandler.getURLSpecFromFile(blogFile);
    gBlogFile = furl;

    gBlogRDF = new RDFDataSource(furl);
    return gBlogRDF;
}

/////////////////////////////////////////////////////////////////
// RDF Helpers
/////////////////////////////////////////////////////////////////

function flock_blogGetAPI(acct) {
    var ds = flock_blogGetDS();
    var profile = ds.getNode(acct);
    var u = profile.getTarget(R2_BLOG_NS+"username").getValue();
    var p = profile.getTarget(R2_BLOG_NS+"password").getValue();
    var t = profile.getTarget(R2_BLOG_NS+"title").getValue();

    var api = profile.getTarget(R2_BLOG_NS+"api");
    var apiname = api.getTarget(R2_BLOG_NS+"apiname").getValue();
    var url = api.getTarget(R2_BLOG_NS+"uri").getValue();
    var blogid = api.getTarget(R2_BLOG_NS+"blogid").getValue();

    var B = null;

    switch(apiname)
    {
        case "atom":
            B = new AtomAPI();
        break;
        case "movabletype":
            B = new MovableTypeAPI();
        break;
        case "blogger":
            B = new BloggerAPI();
        break;
        case "metaweblog":
            B = new MetaweblogAPI();
        break;
        default:
            throw new Error("hmm.  why didn't we find an api?");
        break;
    }

    B.init(apiname, "", url, blogid, "", "");
    B.setAuth(u,p);

    return B;
}

function flock_blogSavePostTags(uri, tags) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    post.addTargetOnce(R2_BLOG_NS+"tags", tags + "");
    ds.save();
}

function flock_blogPostTags(uri) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    if(!post.getTarget(R2_BLOG_NS+"tags")) return "";
    return post.getTarget(R2_BLOG_NS+"tags").getValue();;
}

function flock_blogBlogThisTemplate() {
    var ds = flock_blogGetDS();
    var post = ds.getNode(BLOG_URN_ROOT + "blogthistemplate");
    if(!post.getTarget(R2_BLOG_NS+"data")) return "";
    return post.getTarget(R2_BLOG_NS+"data").getValue();;
}

function flock_blogSaveBlogThisTemplate(data) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(BLOG_URN_ROOT + "blogthistemplate");
    post.addTargetOnce(R2_BLOG_NS+"data", data + "");
    ds.save();
}

function flock_blogWebSite(uri) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    try
    {
        if(!post.getTarget(R2_BLOG_NS+"blogurl")) return null;
    }
    catch(e)
    {
        return null;
    }
    return post.getTarget(R2_BLOG_NS+"blogurl").getValue();;
}


function flock_blogPostCount(uri) {
    var ds = flock_blogGetDS();
    var profile = ds.getNode(uri);

    var posts = profile.getTarget(R2_BLOG_NS+"posts");
    var count = 0;
    var postslist = posts.getChildren();

    while(postslist.hasMoreElements())
    {
        var post = postslist.getNext();
        ++count;
    }
    return count;
}

function flock_blogAccountCount() {
    var ds = flock_blogGetDS();
    var settings = ds.getNode("urn:r2:blog:settings");
    var profiles = settings.getChildren();

    var count = 0;
    while(profiles.hasMoreElements())
    {
        var post = profiles.getNext();
        ++count;
    }
    return count;
}

function flock_blogSavePost(uri, title, html, state, date, dontfix) {
    uri.match(/(.+:posts):(.*)/);
    var container = RegExp.$1;
    var postid = RegExp.$2;

    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    if(!title) title = "Untitled";
    post.addTargetOnce(R2_BLOG_NS+"title", title + "");
    post.addTargetOnce(R2_BLOG_NS+"postid", postid + "");
    if(date) 
    {
        post.addTargetOnce(R2_BLOG_NS+"date", date + "");
        var dateUI = padDateTime(date.getTime());
        post.addTargetOnce(R2_BLOG_NS+"dateUI", dateUI + "");
    }

    if(state) flock_blogSavePostState(uri,state);

    if(html)
    {
        post.addTargetOnce(R2_BLOG_NS+"content", html + "");
    }

    if(uri==BLOG_TMP_POST) return;
    var posts = ds.getNode(container);
    posts.makeSeq();
    posts.addChild(post);


    var allPosts = ds.getNode(BLOG_ALLPOSTS_FOLDER);
    allPosts.makeSeq();
    allPosts.addChild(post);

    ds.save();
}

function flock_blogSavePostState(postid,state) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(postid);
    post.addTargetOnce(R2_BLOG_NS+"state", state);
    var stateUI = "error";
    if(state==BLOG_STATE_OK)
    {
        stateUI = "Published";
    }
    if(state==BLOG_STATE_CHANGED)
    {
        stateUI = "Modified";
    }
    if(state==BLOG_STATE_NEW)
    {
        stateUI = "Draft";
    }
    post.addTargetOnce(R2_BLOG_NS+"stateUI", stateUI);
    ds.save();
}

function flock_blogSavePostTags(uri,tags) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    post.addTargetOnce(R2_BLOG_NS+"tags", tags);
    ds.save();
}

function flock_blogSavePubState(uri,pubstate) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    post.addTargetOnce(R2_BLOG_NS+"pubstate", pubstate + "");
    ds.save();
}

function flock_blogPubState(uri) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);

    try
    {
        return post.getTarget(R2_BLOG_NS+"pubstate").getValue();
    }
    catch(e)
    {
        return "0";
    }
}

function flock_blogPostState(uri) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    return post.getTarget(R2_BLOG_NS+"state").getValue();
}

function flock_blogPostDate(uri) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    return post.getTarget(R2_BLOG_NS+"date").getValue();
}

function flock_blogRemoveSettings(uri) {
    flock_blogClearAllPosts();

    var ds = flock_blogGetDS();


    var profile = ds.getNode(uri);
    ds.deleteRecursive(profile);
    var settings = ds.getNode("urn:r2:blog:settings");
    settings.removeChild(profile);

    flock_blogRebuildAllPosts();

    ds.save();
}

function flock_blogRemoveNotification(url) {
    var ds = flock_blogGetDS();
    var root = ds.getNode(BLOG_PREF_ROOT);
    var node = ds.getNode(BLOG_PREF_ROOT + ":" + url);
    ds.deleteRecursive(node);
    root.removeChild(node);
    ds.save();
}

function flock_blogAddNotification(url) {
    var ds = flock_blogGetDS();
    var root = ds.getNode(BLOG_PREF_ROOT);
    root.makeSeq();
    var node = ds.getNode(BLOG_PREF_ROOT + ":" + url);
    node.addTargetOnce(R2_BLOG_NS + "url", url);
    root.addChild(node);
    ds.save();
}

function flock_blogAddAccount(title, username, password, dontclear, blogurl) {
    var ds = flock_blogGetDS();
    var settings = ds.getNode("urn:r2:blog:settings");
    var count = settings.getChildCount();

    var bloguri = "urn:r2:blog:settings:" + title;
    if(!dontclear) flock_blogRemoveSettings(bloguri);

    var profile = ds.getNode("urn:r2:blog:settings:" + title);
    var api = ds.getNode("urn:r2:blog:settings:" + title + ":" + "api");

    var posts = ds.getNode("urn:r2:blog:settings:" + title + ":" + "posts");
    posts.makeSeq();
    profile.addTargetOnce(R2_BLOG_NS+"title", title + "");
    profile.addTargetOnce(R2_BLOG_NS+"username", username + "");
    profile.addTargetOnce(R2_BLOG_NS+"password", password + "");
    profile.addTargetOnce(R2_BLOG_NS+"api", api);
    profile.addTargetOnce(R2_BLOG_NS+"posts", posts);
    if(blogurl) profile.addTargetOnce(R2_BLOG_NS+"blogurl", blogurl);
    profile.makeSeq();
    settings.addChild(profile);
    ds.save();
}


function flock_blogAddAPI(title, apiname, uri, blogid) {
    var ds = flock_blogGetDS();
    var api = ds.getNode("urn:r2:blog:settings:" + title + ":" + "api");
    api.addTargetOnce(R2_BLOG_NS+"apiname", apiname + "");
    api.addTargetOnce(R2_BLOG_NS+"uri", uri + "");
    api.addTargetOnce(R2_BLOG_NS+"blogid", blogid + "");
    ds.save();
}

function flock_blogInitRDF() {
    var ds = flock_blogGetDS();
    var root=ds.getNode("urn:r2:blog");
    var settings = ds.getNode("urn:r2:blog:settings");
    settings.makeSeq();
    flock_blogAddAccount("Drafts", "", "", true);
    var firstRun = flock_getIntPref("flock.blog.firstRun");
    if(!firstRun)
    {
        flock_blogAddNotification("http://www.technorati.com")
        flock_setIntPref("flock.blog.firstRun", 1);
        flock_setBoolPref("blog.notify", true);
    }

    var allPosts = ds.getNode(BLOG_ALLPOSTS_FOLDER);
    allPosts.makeSeq();

    ds.save();
}

function flock_blogClearAllPosts() {
    try
    {
    	var ds = flock_blogGetDS();
        var dsource = flock_blogGetDS().getRawDataSource();
        var node = ds.getNode(BLOG_ALLPOSTS_FOLDER).source;
        var props=ds.getRawDataSource().ArcLabelsOut(node);
        while(props.hasMoreElements()){
            var prop=props.getNext();
            var target=dsource.GetTarget(node,prop,true);
            dsource.Unassert(node,prop,target)
        }

    } catch(e) {}
}

function flock_blogRebuildAllPosts() {
    var ds = flock_blogGetDS();
    // Erase the allPosts aggregation
    flock_blogClearAllPosts();

    var allPosts = ds.getNode(BLOG_ALLPOSTS_FOLDER);
    allPosts.makeSeq();
    var posts = allPosts.getChildren()
    while(posts.hasMoreElements())
    {
        var post = posts.getNext();
        allPosts.removeChild(post);
    }

    var settings = ds.getNode("urn:r2:blog:settings");
    var profiles = settings.getChildren();
    while(profiles.hasMoreElements())
    {
        var profile = profiles.getNext();
        var list = profile.getTarget(R2_BLOG_NS+"posts");
        var enum = list.getChildren();
        while(enum.hasMoreElements())
        {
            var post = enum.getNext();
            allPosts.addChild(post);
        }
    }
}

function flock_blogDeletePost(uri) {
    var ds = flock_blogGetDS();
    var post = ds.getNode(uri);
    ds.deleteRecursive(post);

    uri.match(/(.+):(.*)/);
    var containeruri = RegExp.$1;
    container = ds.getNode(containeruri);
    container.removeChild(post);

    var allPosts = ds.getNode(BLOG_ALLPOSTS_FOLDER);
    allPosts.removeChild(post);
    ds.save();
}

/////////////////////////////////////////////////////////////////
// Other Helpers
/////////////////////////////////////////////////////////////////

function padDate(str) {
    str = str + "";
    if(str.length==1) return "0" + str;
    return str;
}

function padDateTime(str) {
    str = str + "";
    var diff = 16 - str.length;
    for(var i=0;i<diff;++i)
    {
        str = "0" + str;
    }
    return str;
}

function hexEncode(string) {
    string = String(string); // just to be sure
    var arr = [];
    for (var i=0 ; i<string.length ; i++) {
        arr[i]=toHexCode(string.charCodeAt(i));
    }
    return arr.join("");
}

function toHexCode(number) {
    var string = number.toString(16);
    if (string.length>2) { // pad with zeros to 2 or 4 positions
        return "\\u"+("00"+string).substring(string.length-2);
    } 
    else {
        return "\\x"+("00"+string).substring(string.length);
    }
}

function hexDecode(string) { // assumes format "\xDD\uDDDD\xDD"
    var arr = [];
    var re = /\\[ux]([\da-f]{2,4})/gi;
    while (match = re.exec(string)) {
        arr[arr.length] = String.fromCharCode(parseInt(match[1],16));
    }
    return arr.join("");
}

function flock_blogUpdateCommands(aSet, aController) {
    for(var i=0;i<aSet.childNodes.length;++i)
    {
        var command = aSet.childNodes.item(i);
        var controller = aController;
        if(controller.isCommandEnabled(command.id))
        {
            command.removeAttribute("disabled");
        }
        else
        {
            command.setAttribute("disabled", true);
        }
    }
}

var flock_blogRefreshAccountCommand =
{
    isEnabled: function()
    {
        if(gSelectedBlog && gSelectedBlog.match(BLOG_DRAFTS_FOLDER2)) return false;
        if(!gSelectedBlog) return false;
        return true;
    },
    exec: function()
    {
        var ds = flock_blogGetDS();
        var uri = gSelectedBlog + ":" + "posts";
        var posts = ds.getNode(uri);
        var postslist = posts.getChildren();
        var toDelete = new Array();
        var modCount = 0;
        while(postslist.hasMoreElements())
        {
            var post = postslist.getNext();
            state = post.getTarget(R2_BLOG_NS+"state").getValue();
            if(state!=BLOG_STATE_NEW)
            {
                toDelete[toDelete.length] = post;
            }
            if(state==BLOG_STATE_CHANGED)
            {
                ++modCount;
            }
        }
        if(modCount)
        {
            if(!confirm("You have made local changes to some already published posts. \n\n Updating will delete these changes.  Are you sure you would like to continue?" ))
            {
                return;
            }
        }
        for(var i=0;i<toDelete.length;++i)
        {
            ds.deleteRecursive(toDelete[i]);
            posts.removeChild(toDelete[i]);
        }
        ds.save();

        gSelectedBlog.match(/.+:(.+)/);
        var title = RegExp.$1;
        window.openDialog("blogDownloadPosts.xul", "downloadPostsAccount", "modal", title); 

        ds.save();
    },
}



function flock_blogRefreshList() {
    if(!gRefreshList) return;
    if(gRefreshList.length==0)
    {
        gRefreshList=null;
        return;
    }

    var nextAccount = gRefreshList.pop();
    var listener = new flock_blogPostListingListener(nextAccount, true);
    var B = flock_blogGetAPI(nextAccount);
    B.getRecentPosts(listener, BLOG_POST_DOWNLOAD_COUNT);
}

var flock_blogRefreshAllAccountsCommand =
{
    isEnabled: function()
    {
        return gRefreshList==null;

    },
    exec: function()
    {
        gRefreshList = new Array();
        var ds = flock_blogGetDS();
        var settings = ds.getNode("urn:r2:blog:settings");
        var accounts = settings.getChildren();
        while(accounts.hasMoreElements())
        {
            var account = accounts.getNext();
            var a = BLOG_URN_ROOT + account.getTarget(R2_BLOG_NS+"title").getValue();
            if(a==BLOG_DRAFTS_FOLDER2) continue;

            gRefreshList[gRefreshList.length] = a;
        }
        flock_blogRefreshList();
    },
}



function flock_blogPostListingListener(selectedBlog, force_update, aOnFinish) {
    this.selectedBlog = selectedBlog;
    this.force_update = force_update;
    this.onfinish = aOnFinish;
}

flock_blogPostListingListener.prototype = 
{
    selectedBlog: null,
    force_update: null,
    didUpdate: false,
    onResult: function(result) 
    {
        if(!result)
        {
            alert("returning on parser error");
            return;
        }
        var data = result;
        for(var i=0;i<data.length;++i)
        {
            var posturi = this.selectedBlog + ":posts:" + data[i].postid;
            //alert(posturi);
            var date = new Date(data[i].dateCreated);

            if(!this.force_update && postExists(posturi)) {
                continue;
            }

            if(data[i].dateCreated) {
            }

            var mycontent = "";
            if(data[i].content)
            {
                mycontent = data[i].content;
            }

            if(data[i].description) {
                if(mycontent.length < data[i].description.length) mycontent = data[i].description;
            }

            if(data[i].mt_text_more)
            {
                mycontent += data[i].mt_text_more;
            }

            if(!mycontent) {
                continue;
            }
            mycontent = mycontent.replace(/\r/g,"");
            mycontent = mycontent.replace(/\n/g,"<br/>");
            var title = data[i].title;
            try
            {
                var postid = data[i].postid;
                var uri = this.selectedBlog + ":posts:" + postid;
                flock_blogDeletePost(uri);
                flock_blogSavePost(uri,title,mycontent,BLOG_STATE_OK, date, true);
            }
            catch(e)
            {
                alert(e + e.fileName + " " + e.lineNumber + "\n");
            }
        }
        var ds = flock_blogGetDS();
        var bloguri = this.selectedBlog;
        var blog = ds.getNode(bloguri);
        blog.addTargetOnce(R2_BLOG_NS+"init", 1 + "");
        ds.save();
        this.doUpdate();
     },

     onFault: function (fault)
     {
        alert("There was an error retrieving your blog posts from your service.\n Please check your account information and try again");
        this.doUpdate();
     },

     onError: function (error)
     {
        alert("There was an error retrieving your blog posts from your service.\n It's probably the case that you entered incorrect blog settings (like the API url)");
        this.doUpdate();
     },
     doUpdate: function (error)
     {
        if(this.didUpdate) return;
        didUpdate = true;
        flock_blogRefreshList();
     },
};
