//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const FLOCK_BLOG_TOPBAR_POSTID_FLAVOUR = "flock_blog_post_id";

var gSelectedPost = null;
var gBlogCommandController = null;

function flock_blogBuildPostURI(postid) {
    var url = "chrome://browser/content/flock/blog/blogEditor.xul"
    var params = "?postid=" + hexEncode(postid);
    url += params;
    return url;
}

function flock_blogTopbarUnload() {
    flock_blogRemoveEditorCommands();
    var ds = flock_blogGetDS();
    ds.getRawDataSource().RemoveObserver(gBlogTopbarRDFObserver);
}

function flock_blogTopbarRDFObserve()
{
}

function flock_blogTopbarInitUI() {
    var ds = flock_blogGetDS();

    var postbox = $("flock-blog-topbar-posts");
    postbox.database.AddDataSource(ds.getRawDataSource());
    postbox.ref = BLOG_ALLPOSTS_FOLDER;
    postbox.builder.rebuild();

    flock_blogTopbarUpdateDeck();

    ds.getRawDataSource().RemoveObserver(gBlogTopbarRDFObserver);
    ds.getRawDataSource().AddObserver(gBlogTopbarRDFObserver);
}

function flock_blogTopbarLoad() {
    try {
        flock_blogInit();
        flock_blogRebuildAllPosts();
        flock_blogSetupTopbarCommands();
        flock_blogTopbarInitUI();

        window.addEventListener('focus',function () { window.document.commandDispatcher.updateCommands('focus'); },false);
        window.addEventListener('select',function () { window.document.commandDispatcher.updateCommands('select'); },false);
        window.focus();
        window.document.commandDispatcher.updateCommands('focus');
    }
    catch(e) {
        alert(e);
    }
}

function flock_blogDownloadPostsIfNeedBe() {
    var ds = flock_blogGetDS();
    var settings = ds.getNode("urn:r2:blog:settings");
    var profiles = settings.getChildren();
    var needRefresh = false;

    while(profiles.hasMoreElements())
    {
        var profile = profiles.getNext();
        var title = profile.getTarget(R2_BLOG_NS+"title").getValue();
        if(title==BLOG_DRAFTS_NAME) continue;
        var init = profile.getTarget(R2_BLOG_NS+"init");
        if(!init || init.getValue()!="1")
        {
            needRefresh = true;
        }
    }

    if(needRefresh)
    {
        flock_blogRefreshAllAccountsCommand.exec();
    }
}

function flock_blogTopbarUpdateDeck() {
    var accountCount = flock_blogAccountCount();
    if (accountCount<2) {
        $("flock_blogTopbarDeck").selectedIndex = 0;
    }
    else {
        $("flock_blogTopbarDeck").selectedIndex = 1;
    }

    flock_blogDownloadPostsIfNeedBe();
}

function flock_blogTopbarSelectAccount(aEvent) {
    var blog_title = aEvent.target.label;

    var blog = BLOG_URN_ROOT + blog_title;
}

function flock_blogTopbarSelectPost(postid) {
    window.document.commandDispatcher.updateCommands('focus');
    gSelectedPost = postid;
}

var flock_blogTopbarDDObserver =
{
    onDragOver: function(aEvent,aFlavor,aSession) {
    },

    onDrop: function(aEvent,aDropData,aSession) {
        flock_goDoCommand("cmd_flock_blogBlogThis");
    },

    onDragStart: function(aEvent, aTransferData, aAction) {
        //var postid = aEvent.target.id;
        var postid = "dummy";
        aTransferData.data = new TransferData();
        aTransferData.data.addDataForFlavour(FLOCK_BLOG_TOPBAR_POSTID_FLAVOUR,postid);
    },

    getSupportedFlavours: function() {
        var flavors = new FlavourSet();
        flavors.appendFlavour("text/html");
			return flavors;
    }
}

/************************************************************************/
// RDF Observer
/************************************************************************/


var gBlogTopbarRDFObserver = 
{
    lastUpdate: 0,
    update: function() {
        var now = new Date();
        now = now.getTime();
        if(now - this.lastUpdate>1000) {
            window.setTimeout("flock_blogTopbarUpdateDeck()",500);
        }
        this.lastUpdate = now;
    },
    onAssert: function(ds, source, predicate, target) {
        if(predicate.Value == (R2_BLOG_NS + "postid")) {
        }
        if(predicate.Value == (R2_BLOG_NS + "posts")) {
            this.update();
        }
    },
    onUnassert: function(ds, source, predicate, target) {
        if(predicate.Value == (R2_BLOG_NS + "postid")) {
        }
        if(predicate.Value == (R2_BLOG_NS + "posts")) {
            this.update();
        }
    },
    onChange: function(ds, source, predicate, oldTarget, target) {
    },
    onMove: function(ds, oldSource, newSource, predicate, target) {
    },
}

/////////////////////////////////////////////////////////////////
// Commands for the UI
/////////////////////////////////////////////////////////////////


var flock_blogTopbarEditPostCommand =
{
    isEnabled: function() {
        return gSelectedPost != null;
    },
    exec: function() {
        var url = flock_blogBuildPostURI(gSelectedPost);
        flock_blogDoOpenPost(url);
    },
}

/*
var flock_blogTopbarConversationsCommand =
{
    isEnabled: function() {
        var blogurl = flock_blogWebSite(gSelectedBlog);
        if(!blogurl) return false;
        return gSelectedBlog != null;
    },
    exec: function() {
        var blogurl = flock_blogWebSite(gSelectedBlog);
        window.open(blogurl);
    },
}

var flock_blogTopbarGotoBlogCommand =
{
    isEnabled: function() {
        var blogurl = flock_blogWebSite(gSelectedBlog);
        if(!blogurl) return false;
        return gSelectedBlog != null;
    },
    exec: function() {
        var blogurl = flock_blogWebSite(gSelectedBlog);
        $$('content').loadURI(blogurl);
    },
}
*/

var flock_blogTopbarDeletePostCommand =
{
    isEnabled: function() {
        return gSelectedPost != null;
    },
    exec: function() {
        gSelectedPost.match(/(.+):posts:(.*)/);
        var postid = RegExp.$2;
        var account = RegExp.$1;

        var deleteListener = {
            onResult: function(result) { 
            },

            onFault: function (fault) { 
            },

            onError: function (errorMsg) { 
            },
        };


        if(account!=BLOG_DRAFTS_FOLDER2) {
            var B = flock_blogGetAPI(account);
            B.deletePost(deleteListener, postid);
        }

        flock_blogDeletePost(gSelectedPost);
        gSelectedPost = null;

    },
}

function flock_blogTopbarRefresh() {
    $("flock_blogTopbarDeck").setAttribute("collapsed", true);
    window.setTimeout('$("flock_blogTopbarDeck").removeAttribute("collapsed");', 10);
}

var flock_blogTopbarCreateAccountCommand =
{
    isEnabled: function() {
        return true;
    },
    exec: function() {
        window.openDialog("blogAccountWizard.xul", "accoountWizard", "modal", ""); 
    },
}


function flock_blogRemoveEditorCommands() {
    top.controllers.removeController(gBlogCommandController);
}


function flock_blogSetupTopbarCommands() {
    if(gBlogCommandController) return;
    gBlogCommandController = new CommandController();

    gBlogCommandController.cmd_flock_blogEditPost = flock_blogTopbarEditPostCommand;
    gBlogCommandController.cmd_flock_blogDeletePost = flock_blogTopbarDeletePostCommand;
    gBlogCommandController.cmd_flock_blogCreateAccount = flock_blogTopbarCreateAccountCommand;
    gBlogCommandController.cmd_flock_blogRefreshAllAccounts = flock_blogRefreshAllAccountsCommand;
    gBlogCommandController.cmd_flock_blogBlogThis = flock_blogBlogThisCommand;
    gBlogCommandController.cmd_flock_blogNewPost = flock_blogNewPostCommand;

    top.controllers.appendController(gBlogCommandController);
}

function flock_blogUpdateTopbarCommands() {
    flock_blogUpdateCommands($("flock_BlogTopbarCommands"), gBlogCommandController);
}
