//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

var CommandController = function () {}
CommandController.prototype = {
    getFunctor: function(aCmd) {
        for (p in this) {
            if (aCmd == p) {
                return this[aCmd];
            }
        }
        return;
    },
    supportsCommand: function(aCmd) {
        return (this.getFunctor(aCmd) != null);
    },
    isCommandEnabled: function(aCmd) {
        return this.getFunctor(aCmd).isEnabled();
    },
    doCommand: function(aCmd) {
        XXX('CALLING COMMAND',aCmd);
        this.getFunctor(aCmd).exec()
    },
    onEvent: function(aEvent) {
        
    }
};

var Functor = function () {}
Functor.prototype = {
    isEnabled: function () {
        return true;
    },
    exec: function () { }
}
/*
    Example Usage:

    function flock_initBookmarkView() {
        var tree = $('flock_bookmarkView');
        
        var treeController = new CommandController();
        
        var cmd_delete = new Functor();
        cmd_delete.isEnabled = function () {
            return (tree.view.selection.count);
        };
        cmd_delete.exec = function () {
            curIndex = tree.view.selection.currentIndex;
            dd(flock_treeGetRow(tree, curIndex));
            alert("you tried to delete something!" + curIndex);
        };
        
        treeController.cmd_delete = cmd_delete;

       tree.controllers.appendController(treeController);
    }
*/

function flock_goUpdateCommandSet(aSet)
{
    XXX('UPDATING COMMANDSET', aSet.id);
    for(var i=0;i<aSet.childNodes.length;++i)
    {
        var command = aSet.childNodes.item(i);
        var controller = top.document.commandDispatcher.getControllerForCommand(command.id);
        //dump("flock_goUpdateCommandSet(aSet) " + command.id + "<>" + controller + "\n");
        if(!controller) continue;
        if(controller.isCommandEnabled(command.id))
        {
            command.removeAttribute("disabled");
        }
        else
        {
            command.setAttribute("disabled", true);
        }
    }
}

function flock_goDoCommand(command)
{
    try {
        var controller = top.document.commandDispatcher.getControllerForCommand(command);
        if ( controller && controller.isCommandEnabled(command))
        controller.doCommand(command);
    }
    catch (e) {
        dump("An error occurred executing the " + command + " command\n" + e + "\n");
    }
}

function flock_goUpdateWindowCommandSet(aSet)
{
    for(var i=0;i<aSet.childNodes.length;++i)
    {
        var command = aSet.childNodes.item(i);
        var controller = window.document.commandDispatcher.getControllerForCommand(command.id);
        dump("flock_goUpdateCommandSet(aSet) " + command.id + "<>" + controller + "\n");
        if(!controller) continue;
        if(controller.isCommandEnabled(command.id))
        {
            command.removeAttribute("disabled");
        }
        else
        {
            command.setAttribute("disabled", true);
        }
    }
}

function flock_goDoWindowCommand(command)
{
    try {
        var controller = window.document.commandDispatcher.getControllerForCommand(command);
        if ( controller && controller.isCommandEnabled(command))
        controller.doCommand(command);
    }
    catch (e) {
        dump("An error occurred executing the " + command + " command\n" + e + "\n");
    }
}
