//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

var flock_treeClicker = function (aTree) {
    this._tree = aTree;
};

flock_treeClicker.prototype = {
    click: function (aEvent) {
        var row = {};
        var col = {};
        var obj = {};
        
        var tree = this._tree;
        var treeBox = tree.treeBoxObject;
        
        // Get the call values
        treeBox.getCellAt(aEvent.clientX, aEvent.clientY, row, col, obj);
        
        // Filter out invalid rows
        if (row.value == -1) {
            return;
        }

        // Filter out invalid columns
        if (!col.value) {
            return;
        }


        // Send to a handler
        XXX(obj.value);
        if (obj.value == "twisty" && aEvent.type == "click") {
            this.onTwisty(tree, row.value, col.value);
        } else if (aEvent.type == "dblclick") {
            this.onDblClick(tree, row.value, col.value);
        } else if (aEvent.type == "click") {
            this.onClick(tree, row.value, col.value);
        }

    },
    onDblClick: function (aTree, aRow, aCol) {
        aTree.view.performActionOnCell("dblclick", aRow, aCol);
        aTree.view.performActionOnRow("dblclick", aRow);
        aTree.view.performAction("dblclick");
    },
    onClick: function (aTree, aRow, aCol) {
        aTree.view.performActionOnCell("click", aRow, aCol);
        aTree.view.performActionOnRow("click", aRow);
        aTree.view.performAction("click");

    },
    onTwisty: function (aTree, aRow, aCol) {
        aTree.view.performActionOnCell("twisty", aRow, aCol);
        aTree.view.performActionOnRow("twisty", aRow);
        aTree.view.performAction("twisty");
    }
};





function flock_treeMouseEvent(aEvent) {
    var row = {};
    var col = {};
    var obj = {};
    
    var tree
    //if (aEvent.target.nodeName == "tree") {
    //    tree = aEvent.target;
    //} else if (aEvent.target.parentNode.nodeName == "tree") {
    if (aEvent.target.parentNode.nodeName == "tree") {
        tree = aEvent.target.parentNode;
    } else {
        return;
    }
    //XXX("TARGET",aEvent.target.nodeName);
    //XXX("TREE",tree.nodeName);
    var treeBox = tree.treeBoxObject;
    //dd(tree);
    // Get the call values
    treeBox.getCellAt(aEvent.clientX, aEvent.clientY, row, col, obj);
    
    if (row.value == -1) {
        //XXX("INVALID ROW");
        return;
    }

    if (!col.value) {
        return;
    }

    // Punt to the regular handlers for twisties
    if (obj.value == "twisty" && aEvent.type == "click") {
        return;
    }
    // ... and dblclicks
    if (aEvent.type == "dblclick") {
        tree.view.toggleOpenState(row.value);
        return;
    }
    
    

    var treeCell = flock_treeGetCell(tree, row.value, col.value.index);
    if (!treeCell) {
        return;
    }
    
    var newEvent = document.createEvent("MouseEvent");
    newEvent.initMouseEvent(
        aEvent.type,
        aEvent.canBubble,
        aEvent.cancelable,
        aEvent.view,
        aEvent.detail,
        aEvent.screenX,
        aEvent.screenY,
        aEvent.clientX,
        aEvent.clientY,
        aEvent.ctrlKey,
        aEvent.altKey,
        aEvent.shiftKey,
        aEvent.metaKey,
        aEvent.button,
        treeCell
    );

    treeCell.dispatchEvent(newEvent);
    
}

function flock_treeGetCell(aTree, aRowIndex, aColIndex) {
    var treeRow = flock_treeGetRow(aTree, aRowIndex);
    if (!treeRow) return;

    if (treeRow.childNodes.length <= aColIndex) {
        return;
    }

    var treeCell = treeRow.childNodes[aColIndex];
    return treeCell;
}

function flock_treeGetRow(aTree, aRowIndex) {
    var treeItem = aTree.view.getItemAtIndex(aRowIndex);
    
    if (!treeItem.hasChildNodes()) {
        return;
    }
    var treeRow = treeItem.firstChild;
    return treeRow;
}



var flock_treeBuilderObserver = function () {};
flock_treeBuilderObserver.prototype = {
    DROP_BEFORE: -1,
    DROP_ON: 0,
    DROP_AFTER: 1, 

    /**
     * Methods used by the drag feedback code to determine if a drag is allowable at
     * the current location. To get the behavior where drops are only allowed on
     * items, such as the mailNews folder pane, always return false whe
     * the orientation is not DROP_ON.
     */
    // boolean canDrop(in long index, in long orientation);
    canDrop: function (aIndex, aOrientation) { },
    
    /**
     * Called when the user drops something on this view. The |orientation| param
     * specifies before/on/after the given |row|.
     */
    //void onDrop(in long row, in long orientation);
    onDrop: function (aRow, aOrientation) { },
 
    /** 
     * Called when an item is opened or closed. 
     */
    //void onToggleOpenState (in long index);
    onToggleOpenState: function (aIndex) { },

    /** 
	 * Called when a header is clicked.
     */
    //void onCycleHeader(in wstring colID, in nsIDOMElement elt);
    onCycleHeader: function (aCol, aElement) { },

    /**
     * Called when a cell in a non-selectable cycling column (e.g. 
     * unread/flag/etc.) is clicked.
     */
    //void onCycleCell(in long row, in wstring colID);
    onCycleCell: function (aRow, aCol) { },

    /** 
     * Called when selection in the tree changes
     */
    //void onSelectionChanged();
    onSelectionChanged: function () { },

    /**
     * A command API that can be used to invoke commands on the selection.  
     * The tree will automatically invoke this method when certain keys 
     * are pressed.  For example, when the DEL key is pressed, performAction 
     * will be called with the "delete" string. 
     */
    //void onPerformAction(in wstring action);
    onPerformAction: function (aAction) { },

    /**
     * A command API that can be used to invoke commands on a specific row.
     */
    //void onPerformActionOnRow(in wstring action, in long row);
    onPerformActionOnRow: function (aAction, aRow) { },

    /**
     * A command API that can be used to invoke commands on a specific cell.
     */
    //void onPerformActionOnCell(in wstring action, in long row, in wstring colID);    
    onPerformActionOnCell: function (aAction, aRow, aCol) { }
    
}
