// vim: foldmethod=marker
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const FLOCK_FAVORITES_MAX_AUTH_ERRORS = 3;
const FLOCK_FAVORITES_STATUS_CLEAR_IVAL = 20000;
const FLOCK_FAVORITES_NETWORK_ERROR_MESSAGE = "Error connecting to service";

//flock_initWebServiceListener();
top.addEventListener('load', flock_initWebServiceListener, false);
top.addEventListener('unload', flock_unInitWebServiceListener, false);
var gFavoritesListener = null;

function flock_unInitWebServiceListener () {
    var flockFavorites = flock_getFavoritesService();
    flockFavorites.removeProgressListener(gFavoritesListener);
}

function flock_clearFavoritesStatus()
{
    window.setTimeout("flock_clearFavoritesStatus()", FLOCK_FAVORITES_STATUS_CLEAR_IVAL);
    $("flock-favorites-status-label").value = "";
    $("flock-favorites-status").setAttribute("collapsed", true);
}

function flock_initWebServiceListener () {
    window.setTimeout("flock_clearFavoritesStatus()", FLOCK_FAVORITES_STATUS_CLEAR_IVAL);

    gFavoritesListener = new Object ();
    gFavoritesListener.errorCount = 0;
    gFavoritesListener.setError = function (aSuccess, aMsg) {
        $("flock-favorites-status-label").value = FLOCK_FAVORITES_NETWORK_ERROR_MESSAGE;
        $("flock-favorites-status").removeAttribute("collapsed");
        //Authentication error!
        if(aMsg=="401") {
            if(++this.errorCount>= FLOCK_FAVORITES_MAX_AUTH_ERRORS) {
                var flockFavorites = flock_getFavoritesService();
                flockFavorites.setWebServiceEnabled(false);
                this.errorCount = 0;
                if(flock_mostRecentNavigator()==top) {
                    alert("Oops!\n\nTo use online bookmarks, you must first set up an online bookmarks account.  \n\nAfter setting up an account, you may enable bookmarks sharing through your Web Service Preferences.");
                }
            }
        }
    }
    gFavoritesListener.onSyncUp = function (aSuccess, aMsg) {
        if(!aSuccess) this.setError(aSuccess,aMsg)
        else $("flock-favorites-status-label").value = "Syncing favorites";
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onSyncDown = function (aSuccess, aMsg) {
        if(!aSuccess) this.setError(aSuccess,aMsg)
        else $("flock-favorites-status-label").value = "Synced favorites";
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onCreate = function (aURL, aSuccess, aMsg) {
        if(!aSuccess) this.setError(aSuccess,aMsg)
        else $("flock-favorites-status-label").value = "Created " + aURL;
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onRemove = function (aURL, aSuccess, aMsg) {
        if(!aSuccess) this.setError(aSuccess,aMsg)
        else $("flock-favorites-status-label").value = "Removed " + aURL;
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onUpdate = function (aURL, aSuccess, aMsg) {
        if(!aSuccess) this.setError(aSuccess,aMsg)
        else $("flock-favorites-status-label").value = "Updated " + aURL;
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onSyncUpStart = function () {
        $("flock-favorites-status-label").value = "Syncing favorites";
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onSyncDownStart = function () {
        $("flock-favorites-status-label").value = "Syncing favorites";
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onCreateStart = function (aURL) {
        $("flock-favorites-status-label").value = "Creating " + aURL;
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onRemoveStart = function (aURL) {
        $("flock-favorites-status-label").value = "Removing " + aURL;
        $("flock-favorites-status").removeAttribute("collapsed");
    }
    gFavoritesListener.onUpdateStart = function (aURL) {
        $("flock-favorites-status-label").value = "Updating " + aURL;
        $("flock-favorites-status").removeAttribute("collapsed");
        this.errorCount = 0;
    }

    var flockFavorites = flock_getFavoritesService();
    flockFavorites.addProgressListener(gFavoritesListener);
}

function flock_getDataFromDragSession (aDragSession) {/*{{{*/
    var trans = Components.classes['@mozilla.org/widget/transferable;1'].createInstance(Components.interfaces.nsITransferable);
    //trans.addDataFlavor("moz/rdfitem");
    trans.addDataFlavor("text/x-moz-url");
    trans.addDataFlavor("text/unicode");
    var uri, title;
    
    var arr = [];

    for (var i = 0; i < aDragSession.numDropItems; ++i) {
        var bestFlavour = {};
        var dataObj = {};
        var len = {};
        aDragSession.getData(trans, i);
        trans.getAnyTransferData(bestFlavour, dataObj, len);
        dataObj = dataObj.value.QueryInterface(Components.interfaces.nsISupportsString);
        if (!dataObj) {
            continue;
        }
        dataObj = dataObj.data.substr(0, len.value).split("\n");
        uri = dataObj[0];
        if (dataObj.length > 1 && dataObj[1] != "") {
            title = dataObj[1];
        } else {
            title = uri;
        }
        arr.push({uri: uri, title: title});
    }
    return arr;
}/*}}}*/

function flock_updateFeedsForCurrent() {/*{{{*/
    var flockFavorites = flock_getFavoritesService();
    var flockRSS = flock_getFeedService();
    var curUrl = flock_getCurrentUrl();
    var curFeeds = gBrowser.selectedBrowser.feeds;
    if (curFeeds) {
        var count = {};
        var feedUrls = curFeeds.map(function (x) {return x.href;});
        
        var prioritizeListener = {
            onComplete: function (aCount, aFeeds) {
                XXX('STABBBYY');
                for (var i = 0;i < curFeeds.length;i++) {
                    var f = curFeeds[i];
                    if (aFeeds.indexOf(f.href) != -1) {
                        flockFavorites.addFeed(curUrl, f.href, f.title || f.href);
                        flockRSS.addFeed(f.href);
                    }
                }
            }
        };
        
        
        flock_getFeedService().prioritizeFeeds(feedUrls.length,feedUrls,prioritizeListener);
        // XXX andy: we want to remove the old feeds first in case they have changed
    }

}/*}}}*/


function flock_favoriteCreate(aUrl, aTitle, aDetailed) {
    var star = $('flock_starButton');
    var flockFavorites = flock_getFavoritesService();
    var props = Components.interfaces.flockIFavoriteProperties;
    flockFavorites.createFavorite(
        aUrl,
        {   name: aTitle,
            URL: aUrl,
            fields: props.FIELD_NAME|props.FIELD_URL }
    );

    if (aDetailed || flock_getBoolPref("favorites.addAndTagByDefault")) {
        openDialog("chrome://browser/content/flock/favorites/favoriteProperties.xul",
                   "", "centerscreen,chrome,modal,resizable=no", aUrl);
    }

    flock_favoriteSharingPrompt();
    star.updateState();
}


function flock_filterFeeds(aFeeds) {
    return aFeeds;
}


function flock_inAggMode() {
    if (flock_getCurrentUrl().indexOf('aggview') != -1) {
        return true;
    }
    return false;
}

// dnd handler for toolbar ... hacked form the collections list in the bkmgr/*{{{*/
var flock_collectionDNDObserver = new DNDObserver();
flock_collectionDNDObserver.onDrop = function(aEvent, aXferData, aDragSession) {
    try {
        var flockFavorites = flock_getFavoritesService();
        var props = Components.interfaces.flockIFavoriteProperties;

        var dropTarget = aEvent.target.toolbarView;
        XXX("DROPPED ON", dropTarget.nodeName);

        XXX("DROPPED ON", dropTarget.id);
        var droppedData = flock_getDataFromDragSession(aDragSession);
        for (var i = 0; i < droppedData.length; ++i) {
            var uri = droppedData[i]['uri'];
            var title = droppedData[i]['title'];
            flockFavorites.createFavorite(
                    uri,
                    {   name: title,
                        fields: props.FIELD_NAME }
            );   
            flockFavorites.addFavoriteToCollection(uri,dropTarget.resource);
            if (uri == flock_getCurrentUrl()) {
                flock_updateFeedsForCurrent();
            }
        }
    } catch (e) {
        dd(e)
    }
    return true;
}
flock_collectionDNDObserver.getSupportedFlavours = function () {
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("text/x-moz-url");
    flavourSet.appendFlavour("text/unicode");
    return flavourSet;
}/*}}}*/

// Initialization

// Initialize the plus button and favorites menu
function flock_initStarButton() {
    var star = $('flock_starButton');
    var flockFavorites = flock_getFavoritesService();
    var flockRSS = flock_getFeedService();
    var props = Components.interfaces.flockIFavoriteProperties;

    // Initialize the controller/*{{{*/
    var starController = new CommandController();/*}}}*/
    
    // Base functor for these things/*{{{*/
    var PlusIfValidFunctor = function () {
        var me = this;
        me.isEnabled = function () {
            // Check the current url to be sure it is a real url
            return true;
        };
    };/*}}}*/

        
    // Various functors to implement commands
    var cmd_flock_favoriteCreate = new PlusIfValidFunctor();/*{{{*/
    cmd_flock_favoriteCreate.exec = function () {
        var curUrl = flock_getCurrentUrl();
        var curTitle = flock_getCurrentTitle();
        flock_favoriteCreate(curUrl, curTitle);
        
        //flockFavorites.createFavorite(
        //    curUrl,
        //    {   name: flock_getCurrentTitle(),
        //        URL: curUrl,
        //        fields: props.FIELD_NAME|props.FIELD_URL }
        //);
        //flock_updateFeedsForCurrent();        

        //star.updateState();
        //flock_favoriteSharingPrompt();

    };
    starController.cmd_flock_favoriteCreate = cmd_flock_favoriteCreate;/*}}}*/
    var cmd_flock_favoriteCreateDetailed = new Functor();/*{{{*/
    cmd_flock_favoriteCreateDetailed.exec = function () {
        var curUrl = flock_getCurrentUrl();
        var curTitle = flock_getCurrentTitle();
        flock_favoriteCreate(curUrl, curTitle, true);
        //goDoCommand('cmd_flock_favoriteCreate');
        //var curUrl = flock_getCurrentUrl();
        //openDialog("chrome://browser/content/flock/favorites/favoriteProperties.xul",
        //           "", "centerscreen,chrome,modal,resizable=no", curUrl);

    };    
    starController.cmd_flock_favoriteCreateDetailed = cmd_flock_favoriteCreateDetailed;/*}}}*/
    var cmd_flock_favoriteAddTabsToNewCollection = new Functor();/*{{{*/
    cmd_flock_favoriteAddTabsToNewCollection.isEnabled = function () {
        return ($('content').mTabs.length > 1);
    };
    cmd_flock_favoriteAddTabsToNewCollection.exec = function () {
        // Create a collection
        var collection = flockFavorites.createCollection(
            {   name:'New Collection', 
                fields: props.FIELD_NAME}
        );
        
        // Open Properties on the collection
        openDialog("chrome://browser/content/flock/favorites/collectionProperties.xul",
                   "", "centerscreen,chrome,modal,resizable=no", collection.Value);

        
        // Get all the tabs
        var browsers = $('content').browsers;
        for (var i = 0; i < browsers.length; i++) {
            var webNav = browsers[i].webNavigation;
            var url = webNav.currentURI.spec;
            var name = "";
            try {
                var doc = webNav.document;
                name = doc.title || url;
            } catch (e) {
                name = url;
            }
            var curFeeds = browsers[i].feeds;
            flockFavorites.createFavorite(
                url,
                {   name: name,
                    URL: url,
                    fields: props.FIELD_NAME|props.FIELD_URL }
            );
            if (curFeeds) {
                //dd(curFeeds);
                var count = {};
                var feedUrls = curFeeds.map(function (x) {return x.href;});
                var goodFeeds = flock_getFeedService().prioritizeFeeds(feedUrls.length,feedUrls,count);
                for (var i = 0;i < curFeeds.length;i++) {
                    var f = curFeeds[i];
                    if (goodFeeds.indexOf(f.href) != -1) {
                        flockFavorites.addFeed(url, f.href, f.title || f.href);
                        flockRSS.addFeed(f.href);
                    }
                }
            }
            // Add To Collection
            flockFavorites.addFavoriteToCollection(url, collection);
        }

    };    
    starController.cmd_flock_favoriteAddTabsToNewCollection = cmd_flock_favoriteAddTabsToNewCollection;/*}}}*/
    var cmd_flock_favoriteAddToNewCollection = new Functor();/*{{{*/
    cmd_flock_favoriteAddToNewCollection.exec = function () {
        var curUrl = flock_getCurrentUrl();
        
        // Create a favorite
        goDoCommand('cmd_flock_favoriteCreate');
    
        // Create a collection
        var collection = flockFavorites.createCollection(
            {   name:'New Collection', 
                fields: props.FIELD_NAME}
        );
        
        // Add To Collection
        flockFavorites.addFavoriteToCollection(curUrl, collection);
    
        // Open Properties On The Collection
        
        openDialog("chrome://browser/content/flock/favorites/collectionProperties.xul",
                   "", "centerscreen,chrome,modal,resizable=no", collection.Value);

    };    
    starController.cmd_flock_favoriteAddToNewCollection = cmd_flock_favoriteAddToNewCollection;/*}}}*/
    var cmd_flock_favoriteManage = new Functor();/*{{{*/
    cmd_flock_favoriteManage.exec = function () {
        loadURI('chrome://browser/content/flock/favorites/favoritesManager.xul');
    };    
    starController.cmd_flock_favoriteManage = cmd_flock_favoriteManage;/*}}}*/
    var cmd_flock_webserviceView = new Functor();/*{{{*/
    cmd_flock_webserviceView.exec = function () {
        if (flockFavorites.webService) {
            var url = flockFavorites.webService.getWebView ();
            loadURI (url);
        }
    };    
    starController.cmd_flock_webserviceView = cmd_flock_webserviceView;/*}}}*/

    // Add all the controllers to the right places/*{{{*/
    window.controllers.appendController(starController);/*}}}*/

    // XXX andy: Would set focus here, except that since it is being called 
    // from the XBL file some weird ass shit is happening... dunno man.
    //window.focus()

    //flock_initFeedButton();
}



function flock_initFeedButton() {
    XXX('INIT FEED BUTTON');
    var button = $('flock_feedButton');
    var feedService = flock_getFeedService();
    
    var viewController = new CommandController();

    var ViewToggleIfAllowedFunctor = function () {
        var me = this;
        me.isEnabled = function () {
            var mode = button.getAttribute('mode');
            if (mode == "web") {
                var curFeeds = flock_getCurrentFeeds();
                return (curFeeds);
            } else if (mode == "agg") {
                //var backLink = $('rss_web_link');
                //if (backLink.href) {
                //    return true;
                //}
                return false;
            }
        };
    };

    var cmd_flock_viewToggleAgg = new ViewToggleIfAllowedFunctor();
    cmd_flock_viewToggleAgg.exec = function () {
        XXX('TOGGLE AGGG');
        var feeds = flock_getCurrentFeedsArray();
        var goodFeeds = flock_filterFeeds(feeds);
        
        flock_gotoAgg(goodFeeds,flock_getCurrentUrl());       
    };
    viewController.cmd_flock_viewToggleAgg = cmd_flock_viewToggleAgg;

    // Add all the controllers to the right places/*{{{*/
    window.controllers.appendController(viewController);/*}}}*/
}


//function flock_updateFeedButtonMode() {
//    var feedButton = $('flock_feedButton');
//    if (flock_inAggMode()) {
//        feedButton.setAttribute('aggmode','true');
//    } else {
//        feedButton.removeAttribute('aggmode');
//    }
//}

