// vim: ts=4 sw=4 expandtab cindent foldmethod=marker
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// CONSTANTS/*{{{*/
//Interfaces/*{{{*/
const DEBUG = false;            // set to false to suppress debug messages
const FAVORITES_CID             = Components.ID('{3606f63a-f874-42fa-8174-cc9a0dd0a712}');

const nsISupports               = Components.interfaces.nsISupports;
const nsIFactory                = Components.interfaces.nsIFactory;
const nsIProperties             = Components.interfaces.nsIProperties;
const nsILocalFile              = Components.interfaces.nsILocalFile;
const nsIPrefBranch             = Components.interfaces.nsIPrefBranch;
const flockIWebService          = Components.interfaces.flockIWebService;
//const nsIFavoritesService        = Components.interfaces.nsIFavoritesService;
const flockIFavoritesService    = Components.interfaces.flockIFavoritesService;
const flockIFavoriteProperties  = Components.interfaces.flockIFavoriteProperties;
const nsIClassInfo              = Components.interfaces.nsIClassInfo;
const nsIRDFCompositeDataSource = Components.interfaces.nsIRDFCompositeDataSource;
const nsIRDFRemoteDataSource    = Components.interfaces.nsIRDFRemoteDataSource;
const nsIRDFDataSource          = Components.interfaces.nsIRDFDataSource;
const nsIRDFObserver            = Components.interfaces.nsIRDFObserver;
const nsIRDFService             = Components.interfaces.nsIRDFService;
const nsIRDFContainerUtils      = Components.interfaces.nsIRDFContainerUtils;
const nsIRDFContainer           = Components.interfaces.nsIRDFContainer;
const nsIRDFNode                = Components.interfaces.nsIRDFNode;
const nsIRDFResource            = Components.interfaces.nsIRDFResource;
const nsIRDFLiteral             = Components.interfaces.nsIRDFLiteral;
const nsIRDFInt                 = Components.interfaces.nsIRDFInt;
const nsIRDFDate                = Components.interfaces.nsIRDFDate;
const nsIRDFBlob                = Components.interfaces.nsIRDFBlob;
const nsIWindowWatcherService   = Components.interfaces.nsIWindowWatcherService;
const nsITimer                  = Components.interfaces.nsITimer;
/*}}}*/

// contract-ids/*{{{*/
const FAVORITES_CONTRACTID      = '@mozilla.org/rdf/datasource;1?name=flock-favorites';
const COMPOSITE_CONTRACTID      = '@mozilla.org/rdf/datasource;1?name=composite-datasource';
const MEMORY_DS_CONTRACTID      = '@mozilla.org/rdf/datasource;1?name=in-memory-datasource';
const RDFSERVICE_CONTRACTID     = '@mozilla.org/rdf/rdf-service;1';
const RDFCONTUTILS_CONTRACTID   = '@mozilla.org/rdf/container-utils;1';
const CONTAINER_CONTRACTID      = '@mozilla.org/rdf/container;1';
const DIRECTORY_SVC_CONTRACTID  = '@mozilla.org/file/directory_service;1';
const LOCAL_FILE_CONTRACTID     = '@mozilla.org/file/local;1';
const PREFERENCES_CONTRACTID    = '@mozilla.org/preferences-service;1';
const WINDOWWATCHER_CONTRACTID  = '@mozilla.org/embedcomp/window-watcher;1';
const TIMER_CONTRACTID          = '@mozilla.org/timer;1';
/*}}}*/

// services/*{{{*/
const RDFS = Components.classes[RDFSERVICE_CONTRACTID].getService (nsIRDFService);
const RDFCU = Components.classes[RDFCONTUTILS_CONTRACTID].getService (nsIRDFContainerUtils);
const DIRS = Components.classes[DIRECTORY_SVC_CONTRACTID].getService (nsIProperties);
const PREFS = Components.classes[PREFERENCES_CONTRACTID].getService (nsIPrefBranch);
/*}}}*/

// rdf namespaces/*{{{*/
function Namespace (ns) { return function (arg) { return RDFS.GetResource (ns+arg); } }
const W3RDF                     = Namespace ('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
const NSRDF                     = Namespace ('http://home.netscape.com/NC-rdf#');
const WBRDF                     = Namespace ('http://home.netscape.com/WEB-rdf#');
const FLRDF                     = Namespace ('http://flock.com/rdf#');/*}}}*/

// rdf properties/*{{{*/
const RDF_URL                   = NSRDF ('URL');
const RDF_TAGS                  = FLRDF ('tags');
const RDF_TAG                   = FLRDF ('tag');
const RDF_WSID                  = FLRDF ('wsid');
const RDF_NAME                  = NSRDF ('Name');
const RDF_PERMISSIONS           = FLRDF ('permissions');
const RDF_TYPE                  = W3RDF ('type');
const RDF_FAVORITE              = FLRDF ('Favorite');
const RDF_COLLECTION            = FLRDF ('Collection');
const RDF_LIVEMARK              = NSRDF ('Livemark');
const RDF_FEEDURL               = NSRDF ('FeedURL');
const RDF_FEEDENABLED           = FLRDF ('feedEnabled');
const RDF_SHORTCUTURL           = NSRDF ('ShortcutURL');
const RDF_DESCRIPTION           = NSRDF ('Description');
const RDF_NSYNC                 = FLRDF ('nsync');
const RDF_LASTMODIFIEDDATE      = WBRDF ('LastModifiedDate');
const RDF_LASTVISITDATE         = WBRDF ('LastVisitDate');
const RDF_ADDEDDATE             = FLRDF ('FavoriteAddDate');
const RDF_FAVORITESROOT         = FLRDF ('FavoritesRoot');
const RDF_COLLECTIONSROOT       = FLRDF ('CollectionsRoot');
const RDF_TAGSROOT              = FLRDF ('TagsRoot');
const RDF_INSTANCEOF            = W3RDF ('instanceOf');
const RDF_SEQ                   = W3RDF ('Seq');
const RDF_FEED                  = FLRDF ('feed');
const RDF_TOOLBAR               = FLRDF ('toolbar');/*}}}*/

// constants from the idl/*{{{*/
const FIELD_WSID                = flockIFavoriteProperties.FIELD_WSID;
const FIELD_TAGS                = flockIFavoriteProperties.FIELD_TAGS;
const FIELD_PERMISSIONS         = flockIFavoriteProperties.FIELD_PERMISSIONS;
const FIELD_NAME                = flockIFavoriteProperties.FIELD_NAME;
const FIELD_URL                 = flockIFavoriteProperties.FIELD_URL;
const FIELD_SHORTCUTURL         = flockIFavoriteProperties.FIELD_SHORTCUTURL;
const FIELD_DESCRIPTION         = flockIFavoriteProperties.FIELD_DESCRIPTION;
const FIELD_NSYNC               = flockIFavoriteProperties.FIELD_NSYNC;
const FIELD_LASTVISITDATE       = flockIFavoriteProperties.FIELD_LASTVISITDATE;
const FIELD_LASTMODIFIEDDATE    = flockIFavoriteProperties.FIELD_LASTMODIFIEDDATE;
const FIELD_ADDEDDATE           = flockIFavoriteProperties.FIELD_ADDEDDATE;/*}}}*/

// prefs/*{{{*/
const PREF_NAME                 = 'flock.favorites.webservice.name';
const PREF_USERNAME             = 'flock.favorites.webservice.username';
const PREF_PASSWORD             = 'flock.favorites.webservice.password';
const PREF_WEB_SERVICE_ENABLED  = 'flock.favorites.webservice.enabled';
const PREF_FAVORITES_INITIAL    = 'flock.favorites.initialSetup';
/*}}}*/

// urls
/*{{{*/
const FAVORITES_PROGRESS_URL    = 'chrome://browser/content/flock/favorites/favoritesSyncing.xul';
/*}}}*/
/*}}}*/

// Functions to create RDF node objects from javascript types/*{{{*/

function rdfLiteral (s) {
    return RDFS.GetLiteral (s);
}

function rdfDate (d) {
    return RDFS.GetDateLiteral (d);
}

function rdfBoolean (b) {
    return rdfLiteral (b?"true":"false");
}/*}}}*/

// a magic object for managing favorite properties/*{{{*/
var FavoritePropertyMetadata = new Object ();
FavoritePropertyMetadata.list = new Array ();
FavoritePropertyMetadata.list.push ({
    name:        'wsid', 
    rdfProperty: RDF_WSID,
    idlProperty: 'wsid',
    idlMask:     FIELD_WSID,
    sync:        false, // immutable
    idlToRdf:    rdfLiteral,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.list.push ({
    name:        'tags', 
    rdfProperty: RDF_TAGS,
    idlProperty: 'tags',
    idlMask:     FIELD_TAGS,
    sync:        true,
    idlToRdf:    rdfLiteral,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.list.push ({
    name:        'permissions', 
    rdfProperty: RDF_PERMISSIONS,
    idlProperty: 'permissions',
    idlMask:     FIELD_PERMISSIONS,
    sync:        true,
    idlToRdf:    rdfLiteral,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.list.push ({
    name:        'name', 
    rdfProperty: RDF_NAME,
    idlProperty: 'name',
    idlMask:     FIELD_NAME,
    sync:        true,
    idlToRdf:    rdfLiteral,
    rdfToIdl:    function (n) { return n.Value; }
});
/*
FavoritePropertyMetadata.list.push ({
    name:        'URL', 
    rdfProperty: RDF_URL,
    idlProperty: 'URL',
    idlMask:     FIELD_URL,
    sync:        false, // immutable
    idlToRdf:    rdfLiteral,
});
FavoritePropertyMetadata.list.push ({
    name:        'shortcutURL', 
    rdfProperty: RDF_SHORTCUTURL,
    idlProperty: 'shortcutURL',
    idlMask:     FIELD_SHORTCUTURL,
    sync:        false, // does not exist in web services
    idlToRdf:    rdfLiteral,
});
*/
FavoritePropertyMetadata.list.push ({
    name:        'description', 
    rdfProperty: RDF_DESCRIPTION,
    idlProperty: 'description',
    idlMask:     FIELD_DESCRIPTION,
    sync:        true,
    idlToRdf:    rdfLiteral,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.list.push ({
    name:        'lastVisitDate', 
    rdfProperty: RDF_LASTVISITDATE,
    idlProperty: 'lastVisitDate',
    idlMask:     FIELD_LASTVISITDATE,
    sync:        false,
    idlToRdf:    rdfDate,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.list.push ({
    name:        'addedDate', 
    rdfProperty: RDF_ADDEDDATE,
    idlProperty: 'addedDate',
    idlMask:     FIELD_ADDEDDATE,
    sync:        false,
    idlToRdf:    rdfDate,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.list.push ({
    name:        'nsync', 
    rdfProperty: RDF_NSYNC,
    idlProperty: 'nsync',
    idlMask:     FIELD_NSYNC,
    sync:        false, // used in synchronization
    idlToRdf:    rdfBoolean,
    rdfToIdl:    function (n) { return n.Value=="true"; }
});
FavoritePropertyMetadata.list.push ({
    name:        'lastModifiedDate', 
    rdfProperty: RDF_LASTMODIFIEDDATE,
    idlProperty: 'lastModifiedDate',
    idlMask:     FIELD_LASTMODIFIEDDATE,
    sync:        false,
    idlToRdf:    rdfDate,
    rdfToIdl:    function (n) { return n.Value; }
});
FavoritePropertyMetadata.byRdfProperty = function (prop) {
    var value = prop.ValueUTF8;
    for (var i=0; i<this.list.length; i++) {
        if (this.list[i].rdfProperty.ValueUTF8 == value) {
            return this.list[i];
        }
    }
    return null;
}
FavoritePropertyMetadata.repr = function (props) {
    var value = '{ ';
    for (var i=0; i<this.list.length; i++) {
        if (props.fields & this.list[i].idlMask) {
            value = value + this.list[i].idlProperty + ': "'+props[this.list[i].idlProperty]+'", ';
        }
    }
    return value + '}';
}/*}}}*/

// utility functions/*{{{*/

// a toString method for RDF nodes/*{{{*/
function rdf_repr (node) {
        try { 
            var value = node.QueryInterface (nsIRDFResource).ValueUTF8;
            value = value.replace ('http://www.w3.org/1999/02/22-rdf-syntax-ns#', 'W3:');
            value = value.replace ('http://home.netscape.com/NC-rdf#', 'NS:');
            value = value.replace ('http://developer.mozilla.org/rdf/vocabulary/forward-proxy#', 'FP:');
            value = value.replace ('http://flock.com/rdf#', 'FL:');
            return value;
        }
        catch (e) { }

        try { 
            var date = node.QueryInterface (nsIRDFDate);
            return 'date:'+date.Value;
        }
        catch (e) { }

        try { 
            var blob = node.QueryInterface (nsIRDFBlob);
            return 'blob: length'+blob.Length;
        }
        catch (e) { }

        try { 
            var int = node.QueryInterface (nsIRDFInt);
            return 'int:'+int.Value;
        }
        catch (e) { }

        try { 
            var literal = node.QueryInterface (nsIRDFLiteral);
            return '"'+literal.Value+'"';
        }
        catch (e) { }
        
        return 'unknown:'+node;
}/*}}}*/

// dump rdf/*{{{*/
function dump_rdf (data_source) {
    var ser=Components.classes["@mozilla.org/rdf/xml-serializer;1"].createInstance(Components.interfaces.nsIRDFXMLSerializer);
    ser.init(data_source);
    ser.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(
    {
        close : function(){dump("\n");},
        flush : function(){},
        write : function (buffer,count){ dump (buffer); return count; },
        writeFrom : function (stream,count){},
        isNonBlocking: false
    });
}/*}}}*/

// copy all the assertions from one datasource to another/*{{{*/
function copy_ds (ds, ds2) {
    var resources = ds.GetAllResources ();
    while (resources.hasMoreElements ()) {
        var resource = resources.getNext ();
        //dump ('resource: ' + rdf_repr (resource) + '\n');
        var arcs = ds.ArcLabelsOut (resource);
        while (arcs.hasMoreElements ()) {
            var arc = arcs.getNext ().QueryInterface (nsIRDFResource);
            var targets1 = ds.GetTargets (resource, arc, true);
            while (targets1.hasMoreElements ()) {
                var target = targets1.getNext().QueryInterface (nsIRDFNode);
                //dump (' arc1: '+rdf_repr (arc)+' -> '+rdf_repr (target)+'\n');
                ds2.Assert (resource, arc, target, true);
            }
            var targets0 = ds.GetTargets (resource, arc, false);
            while (targets0.hasMoreElements ()) {
                var target = targets0.getNext().QueryInterface (nsIRDFNode);
                //dump (' arc0: '+rdf_repr (arc)+' -> '+rdf_repr (target)+'\n');
                ds2.Assert (resource, arc, target, false);
            }
        }
    }
}/*}}}*/

// generate a graphviz input file/*{{{*/
function rdf_dot (ds) {
    var resources = ds.GetAllResources ();
    dump ('digraph G {\n');
    while (resources.hasMoreElements ()) {
        var resource = resources.getNext ();
        var arcs = ds.ArcLabelsOut (resource);
        while (arcs.hasMoreElements ()) {
            var arc = arcs.getNext ().QueryInterface (nsIRDFResource);
            var targets1 = ds.GetTargets (resource, arc, true);
            while (targets1.hasMoreElements ()) {
                var target = targets1.getNext().QueryInterface (nsIRDFNode);
                //dump ('\t"'+rdf_repr (arc)+'" '+rdf_repr (target)+'\n');
                dump ('\t"'+rdf_repr (resource)+'" -> "'+rdf_repr (target)+'" [label="'+rdf_repr (arc)+'"]\n');
            }
        }
    }
    dump ('}\n');
}/*}}}*//*}}}*/

// Service implementation
function flockFavoritesService()/*{{{*/
{
    dump ('CONSTRUCTOR: flockFavoritesService\n\n\n');
    this.externalListeners = new Array();
    const PROMPTSERVICE_CONTRACTID = "@mozilla.org/embedcomp/prompt-service;1";
    const nsIPromptService = Components.interfaces.nsIPromptService;
    this.promptService =
        Components.classes[PROMPTSERVICE_CONTRACTID].getService(nsIPromptService);

    this.luceneIndexer = Components.classes['@flock.com/lucene/indexer;1'].getService(Components.interfaces.flockILuceneIndexer);
    
    // create a datasource in the profile directory
    try {
        var profd = DIRS.get('ProfD', nsILocalFile).path;
        dump ('profile dir=\"'+profd+'\"\n');
        var file = Components.classes[LOCAL_FILE_CONTRACTID]
            .createInstance(nsILocalFile);
        file.initWithPath(profd);
        file.append('flock_favorites.rdf');
        /*
        if (!file.exists ()) {
            dump ('NO FAVORITES!\n');
            // no favorites!
            file.create (0x00, 0644);
            var transport = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance (Components.interfaces.nsIFileOutputStream);
            transport.init (file, 0x04 | 0x08 | 0x10, 064, 0);
            transport.write (DEFAULT_FAVORITES, DEFAULT_FAVORITES.length);
            transport.close ();

        }
        */
        this.dataSource=RDFS.GetDataSourceBlocking ('file://' + file.path)
            .QueryInterface (nsIRDFRemoteDataSource);
    } catch (e) {
        dump ('EXCEPTION: '+e+'\n');
        // or if we can't, create it in /tmp
        this.dataSource=
            RDFS.GetDataSourceBlocking ('file:///tmp/flock_favorites.rdf')
            .QueryInterface (nsIRDFRemoteDataSource);
    }

    // get references to our favorites root and collections root
    this.favoritesRoot = this.Seq (RDF_FAVORITESROOT);
    this.collectionsRoot = this.Seq (RDF_COLLECTIONSROOT);
    this.tagsRoot = this.Seq(RDF_TAGSROOT);

    // XXX HACK
    PREFS.setCharPref (PREF_NAME, 'delicious');

    // connect to our web service
    try {
        var webservice_name = PREFS.getCharPref (PREF_NAME);
        if (webservice_name && webservice_name.length) {
            dump ('SETTING WEB SERVICE TO: '+webservice_name+'\n');
            this.setWebService (webservice_name);

            if (this.getWebServiceEnabled ()) {
                // push any pending updates up
                this.syncUp ();
                // pull any pending updates down
                this.syncDown ();
            }
        } else {
            dump ('NOT SETTING WEB SERVICE ('+webservice_name+')\n');
        }
    } catch (e) { 
        dump ('NOT SETTING WEB SERVICE:\n'+e+'\n');
    }

    /*
    try {
        var webservice_username = PREFS.getCharPref (PREF_USERNAME);
        var webservice_password = PREFS.getCharPref (PREF_PASSWORD);
        var listener = new Object ();
        listener.onSuccess = function () { dump ('web service authentication success\n'); }
        this.webService.authenticate (webservice_username, webservice_password, listener);
    } catch (e) {
        dump ('NOT LOGGING INTO DELICIOUS\n');
    }
    */
    dump ('flock favorites data source set up\n');

    this.dataSourceSetUp = false;
}/*}}}*/
flockFavoritesService.prototype.SetUpDataSource = function () {/*{{{*/
    // if we've already done this set up then lets not do it again
    if (this.dataSourceSetUp) {
        return;
    }
}/*}}}*/
flockFavoritesService.prototype.Seq = function (node) {/*{{{*/
    // make the resource a sequence if its not and return a container object
    if (!RDFCU.IsContainer (this.dataSource, node)) {
        return RDFCU.MakeSeq (this.dataSource, node);
    }
    var container = Components.classes[CONTAINER_CONTRACTID].createInstance (nsIRDFContainer);
    container.Init (this.dataSource, node);
    return container;
}/*}}}*/
flockFavoritesService.prototype.setProperties = function (aResource, aProperties, aMask) {/*{{{*/
    var mask = aProperties.fields & (aMask?aMask:-1);
    for (var i = 0; i < FavoritePropertyMetadata.list.length; i++) {
        var prop = FavoritePropertyMetadata.list[i];
        if (mask & prop.idlMask) {
            if (prop.name == "tags") {
                this.setTags(aResource.Value, aProperties[prop.idlProperty]);
                // Do Some crazy tag shit
            } 
            // does that property already exist?
            var old_target = this.dataSource.GetTarget (aResource, prop.rdfProperty, true);
            if (old_target) {
                this.dataSource.Change (aResource, prop.rdfProperty, old_target, 
                        prop.idlToRdf (aProperties[prop.idlProperty]));
            } else {
                this.dataSource.Assert (aResource, prop.rdfProperty, 
                        prop.idlToRdf (aProperties[prop.idlProperty]), true);
            }
        }
    }
    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.getProperties = function (aResource) {/*{{{*/
    var properties = new Object ();
    properties.fields = 0;
    for (var i=0; i<FavoritePropertyMetadata.list.length; i++) {
        var property = FavoritePropertyMetadata.list[i].rdfProperty;
        var target = this.dataSource.GetTarget (aResource, property, true);
        if (target == null) {
            continue;
        }
        var idlprop = FavoritePropertyMetadata.list[i].idlProperty
        properties[idlprop] = FavoritePropertyMetadata.list[i].rdfToIdl (target);
        dump ('getProperty - prop: '+idlprop+'\n');
        dump ('            target: '+rdf_repr(target)+'\n');
        dump ('      target.Value: '+target.Value+'\n');
        dump ('       -> rdfToIdl: '+FavoritePropertyMetadata.list[i].rdfToIdl(target)+'\n');
        properties[idlprop] = FavoritePropertyMetadata.list[i].rdfToIdl(target);
        dump ('             value: '+properties[idlprop]+'\n');
        properties.fields |= FavoritePropertyMetadata.list[i].idlMask;
    }
    return properties;
}/*}}}*/
flockFavoritesService.prototype.removeProperties = function (aResource) {/*{{{*/
    for (var i=0; i<FavoritePropertyMetadata.list.length; i++) {
        var property = FavoritePropertyMetadata.list[i].rdfProperty;
        var targets = this.dataSource.GetTargets (aResource, property, true);
        while (targets.hasMoreElements ()) {
            this.dataSource.Unassert (aResource, property, targets.getNext ());
        }
    }
}/*}}}*/


// the nsIClassInfo implementation/*{{{*/
flockFavoritesService.prototype.flags = nsIClassInfo.Singleton;
flockFavoritesService.prototype.classDescription = "Flock Favorites Service";
flockFavoritesService.prototype.getInterfaces = function (count) {
    var interfaceList = [flockIFavoritesService, nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
}
flockFavoritesService.prototype.getHelperForLanguage = function (count) {return null;}/*}}}*/


// the nsISupports implementation/*{{{*/
flockFavoritesService.prototype.QueryInterface =
function (iid) {
    if (!iid.equals(flockIFavoritesService) && 
        !iid.equals(nsIRDFDataSource) &&
        !iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    if (iid.equals(nsIRDFDataSource) && !this.dataSourceSetup) {
        this.SetUpDataSource ();
    }
    return this;
}/*}}}*/


// the nsIRDFDataSource implementation
flockFavoritesService.prototype.URI = 'rdf:flock-favorites';
flockFavoritesService.prototype.GetSource = function (aProperty, aTarget, aTruthValue) { /*{{{*/
    return this.dataSource.GetSource (aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockFavoritesService.prototype.GetSources = function (aProperty, aTarget, aTruthValue) { /*{{{*/
    return this.dataSource.GetSources (aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockFavoritesService.prototype.GetTarget = function (aSource, aProperty, aTruthValue) { /*{{{*/
    return this.dataSource.GetTarget (aSource, aProperty, aTruthValue); 
}/*}}}*/
flockFavoritesService.prototype.GetTargets = function (aSource, aProperty, aTruthValue) { /*{{{*/
    return this.dataSource.GetTargets (aSource, aProperty, aTruthValue); 
}/*}}}*/
flockFavoritesService.prototype.Assert = function (aSource, aProperty, aTarget, aTruthValue) { /*{{{*/
    this.dataSource.Assert (aSource, aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockFavoritesService.prototype.Unassert = function (aSource, aProperty, aTarget) { /*{{{*/
    this.dataSource.Unassert (aSource, aProperty, aTarget); 
}/*}}}*/
flockFavoritesService.prototype.Change = function (aSource, aProperty, aOldTarget, aNewTarget) { /*{{{*/
    this.dataSource.Change (aSource, aProperty, aOldTarget, aNewTarget); 
}/*}}}*/
flockFavoritesService.prototype.Move = function (aOldSource, aNewSource, aProperty, aTarget) { /*{{{*/
    this.dataSource.Move (aOldSource, aNewSource, aProperty, aTarget); 
}/*}}}*/
flockFavoritesService.prototype.HasAssertion = function (aSource, aProperty, aTarget, aTruthValue) { /*{{{*/
    return this.dataSource.HasAssertion (aSource, aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockFavoritesService.prototype.AddObserver = function (aObserver) { /*{{{*/
    this.dataSource.AddObserver (aObserver); 
}/*}}}*/
flockFavoritesService.prototype.RemoveObserver = function (aObserver) { /*{{{*/
    this.dataSource.RemoveObserver (aObserver); 
}/*}}}*/
flockFavoritesService.prototype.ArcLabelsIn = function (aNode) { /*{{{*/
    return this.dataSource.ArcLabelsIn (aNode); 
}/*}}}*/
flockFavoritesService.prototype.ArcLabelsOut = function (aNode) { /*{{{*/
    return this.dataSource.ArcLabelsOut (aNode); 
}/*}}}*/
flockFavoritesService.prototype.GetAllResources = function () { /*{{{*/
    return this.dataSource.GetAllResources (); 
}/*}}}*/
flockFavoritesService.prototype.IsCommandEnabled = function (aSources, aCommand, aArguments) { /*{{{*/
    return this.dataSource.IsCommandEnabled (aSources, aCommand, aArguments); 
}/*}}}*/
flockFavoritesService.prototype.DoCommand = function (aSources, aCommand, aArguments) { /*{{{*/
    this.dataSource.DoCommand (aSources, aCommand, aArguments); 
}/*}}}*/
flockFavoritesService.prototype.GetAllCmds = function (aSource) { /*{{{*/
    return this.dataSource.GetAllCmds (aSource); 
}/*}}}*/
flockFavoritesService.prototype.hasArcIn = function (aNode, aArc) { /*{{{*/
    return this.dataSource.hasArcIn (aNode, aArc); 
}/*}}}*/
flockFavoritesService.prototype.hasArcOut = function (aNode, aArc) { /*{{{*/
    return this.dataSource.hasArcOut (aNode, aArc); 
}/*}}}*/
flockFavoritesService.prototype.beginUpdateBatch = function () { /*{{{*/
    this.dataSource.beginUpdateBatch (); 
}/*}}}*/
flockFavoritesService.prototype.endUpdateBatch = function () { /*{{{*/
    this.dataSource.endUpdateBatch (); 
}/*}}}*/

// the flockIFavoritesService implementation
flockFavoritesService.prototype.createFavorite = function (aURL, aProperties) {/*{{{*/
    dump ('createFavorite ('+aURL+', '+FavoritePropertyMetadata.repr (aProperties)+')\n');
    var resource = RDFS.GetResource (aURL);
    // set the favorite type
    this.dataSource.Assert (resource, RDF_TYPE, RDF_FAVORITE, true);
    // if this doesn't already exist
    if (this.favoritesRoot.IndexOf (resource) == -1) {
    // add this to the favorites root
        this.favoritesRoot.AppendElement (resource);
    }
    // set the properties
    this.setProperties (resource, aProperties);

    // add this document to the full text indexer
    this.luceneIndexer.AddDocument (aURL);

    // if this favorite isn't marked as synced then...
    if (!(aProperties.fields|FIELD_NSYNC && aProperties.nsync)) {
        // mark this favorite as unsynced
        this.setProperties (resource, {nsync: false, fields: FIELD_NSYNC});
        
        // send it to the web service
        if (this.webService && this.getWebServiceEnabled ()) {
            var listener = new Object ();
            listener.resource = resource;
            listener.fsvc = this;
            listener.externalListeners = this.externalListeners;
            listener.onSuccess = function () {
                dump ('web service success\n');
                this.fsvc.setProperties (this.resource, {nsync: true, fields: FIELD_NSYNC});
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onCreate(aURL, true, "ok");
            }
            listener.onError = function (msg) { 
                dump ('web service error: '+msg+'\n');
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onCreate(aURL, false, msg);
            }
            listener.onFault = function (msg) { 
                dump ('web service fault: '+msg+'\n');
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onCreate(aURL, false, msg);
            }
            listener.onStart = function () { 
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onCreateStart(aURL);
            }
            dump ('calling webService.add ('+aURL+')\n');
            this.webService.add (aURL, aProperties, listener);
        }
    }
}/*}}}*/
flockFavoritesService.prototype.removeFavorite = function (aURL) {/*{{{*/
    // FIXME: fail for collections/collections?

    // we need to find the favorite
    var resource = RDFS.GetResource (aURL);

    //this.beginUpdateBatch ();

    dump ('REMOVE FAVORITE CONTENTS\n');
    // remove livemark/feed contents
    if (RDFCU.IsContainer (this.dataSource, resource)) {
        var container = Components.classes[CONTAINER_CONTRACTID].createInstance(nsIRDFContainer);
        container.Init(this.dataSource, resource);
        //var container = Container (this.dataSource, resource);
        var children = container.GetElements ();
        while (children.hasMoreElements ()) {
            var child = children.getNext ();
            container.RemoveElement (child, false);
        }
    }

    dump ('REMOVE FAVORITE TYPE\n');
    // FIXME: move this into the FavoritePropertyMetadata thing somehow
    this.dataSource.Unassert (resource, RDF_TYPE, RDF_FAVORITE);

    dump ('REMOVE FAVORITE PROPERTIES\n');
    // remove the properties
    this.removeProperties (resource);

    dump ('REMOVE FAVORITE FROM ROOT\n');
    this.favoritesRoot.RemoveElement (resource, true)

    dump ('REMOVE FAVORITE FROM PARENTS\n');
    // remove this from its parents
    var arcs = this.dataSource.ArcLabelsIn (resource);
    while (arcs.hasMoreElements ()) {
        var arc = arcs.getNext ();
        var parent = this.dataSource.GetSource (arc, resource, true);
        dump ('PARENT: '+rdf_repr (parent)+'\n');
        if (RDFCU.IsContainer (this.dataSource, parent)) {
            dump ('REMOVING '+rdf_repr (resource)+' from '+rdf_repr (parent)+'\n');
            this.Seq (parent).RemoveElement (resource, true);
        }
    }

    this.dataSource.Flush ();

    // try to delete this document from the full text index.
    // it will fail if the document is still in the history
    if (!this.luceneIndexer.TryToDelete (aURL)) {
        // we need up update the metadata in the history index
        this.luceneIndexer.AddDocument (aURL);
    }

    // send it to the web service
    // FIXME: we need to persist deletes across browser-quit somehow...
    if (this.webService && this.getWebServiceEnabled ()) {
        var listener = new Object ();
        listener.resource = resource;
        listener.fsvc = this;
        listener.externalListeners = this.externalListeners;
        listener.onSuccess = function () {
            dump ('web service success\n');
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onRemove(aURL, true, '');
        }
        listener.onError = function (msg) { 
            dump ('web service error: '+msg+'\n');
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onRemove(aURL, false, msg);
        }
        listener.onFault = function (msg) { 
            dump ('web service fault: '+msg+'\n');
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onRemove(aURL, false, msg);
        }
        listener.onStart = function () { 
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onRemoveStart(aURL);
        }
        this.webService.remove (aURL, listener);
    }
    //this.endUpdateBatch ();
}/*}}}*/
flockFavoritesService.prototype.updateFavorite = function (aURL, aProperties) {/*{{{*/
    dump ('updateFavorite ('+aURL+', '+FavoritePropertyMetadata.repr (aProperties)+')\n');
    // we need to find the favorite
    var resource = RDFS.GetResource (aURL);
    // FIXME: make sure this is a favorite
    this.setProperties (resource, aProperties);

    // add this document to the full text indexer
    this.luceneIndexer.AddDocument (aURL);

    // if this favorite isn't marked as synced then...
    if (!(aProperties.fields|FIELD_NSYNC && aProperties.nsync)) {
        // mark this favorite as unsynced
        this.setProperties (resource, {nsync: false, fields: FIELD_NSYNC});
        
        // send it to the web service
        if (this.webService && this.getWebServiceEnabled ()) {
            var listener = new Object ();
            listener.resource = resource;
            listener.fsvc = this;
            listener.externalListeners = this.externalListeners;
            listener.onSuccess = function () {
                dump ('web service success\n');
                this.fsvc.setProperties (this.resource, {nsync: true, fields: FIELD_NSYNC});
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onUpdate(aURL, true, "ok");
            }
            listener.onError = function (msg) { 
                dump ('web service error: '+msg+'\n');
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onUpdate(aURL, false, msg);
            }
            listener.onFault = function (msg) { 
                dump ('web service fault: '+msg+'\n');
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onUpdate(aURL, false, msg);
            }
            listener.onStart = function () { 
                for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onUpdateStart(aURL);
            }
            //this.webService.change (aURL, aProperties, listener);
            this.webService.add (aURL, aProperties, listener);
        }
    }
}/*}}}*/
flockFavoritesService.prototype.isFavorite = function (aURL) {/*{{{*/
    try {
        var resource = RDFS.GetResource (aURL);
        return this.dataSource.HasAssertion (resource, RDF_TYPE, RDF_FAVORITE, true);
    } catch (e) {
        dump ("couldn't get favoriteness of '"+aURL+"'\n");
        return false;
    }
}/*}}}*/
flockFavoritesService.prototype.getFavoriteProperties = function (aURL) {/*{{{*/
    var resource = RDFS.GetResource (aURL);
    if (!this.dataSource.HasAssertion (resource, RDF_TYPE, RDF_FAVORITE, true)) {
        return null;
    }
    var props = this.getProperties (resource);
    //dump ('flockFavoritesService.getFavoriteProperties ('+aURL+') -> '+FavoritePropertyMetadata.repr (props)+'\n');
    return props;
}/*}}}*/
flockFavoritesService.prototype.createCollection = function (aProperties) {/*{{{*/
    // FIXME: ensure that name is set
    dump ('createCollection ('+aProperties.name+') inside '+rdf_repr (RDF_COLLECTIONSROOT)+'\n');

    // get the resource
    var collection = RDFS.GetAnonymousResource ();

    // make it a sequence
    this.Seq (collection);
    
    // set its type to be collection
    this.dataSource.Assert (collection, RDF_TYPE, RDF_COLLECTION, true);

    // set its properties
    this.setProperties (collection, aProperties, FIELD_NAME|FIELD_DESCRIPTION);
    
    // add this collection to the collections root
    this.collectionsRoot.AppendElement (collection);

    return collection;
}/*}}}*/
flockFavoritesService.prototype.removeCollection = function (aCollection) {/*{{{*/
    dump ('removeCollection ('+rdf_repr(aCollection)+')\n');
    //this.beginUpdateBatch ();

    dump ('REMOVING COLLECTION CHILDREN\n');
    // empty it
    var container = this.Seq (aCollection);
    var elements = container.GetElements ();
    while (elements.hasMoreElements ()) {
        container.RemoveElement (elements.getNext (), false);
    }

    dump ('REMOVING COLLECTION PROPERTIES\n');
    // remove the properties
    this.removeProperties (aCollection);

    dump ('REMOVE COLLECTION TYPE\n');
    // FIXME: move this into the FavoritePropertyMetadata thing somehow
    this.dataSource.Unassert (aCollection, RDF_TYPE, RDF_COLLECTION);

    dump ('REMOVING COLLECTION FROM COLLECTIONSROOT\n');
    // remove this collection from the root
    this.collectionsRoot.RemoveElement (aCollection, true);

    dump ('REMOVE RDF CRAP\n');
    this.dataSource.Unassert (aCollection, RDF_INSTANCEOF, RDF_SEQ);
  
     //this.endUpdateBatch ();
    this.dataSource.Flush ();
    dump ('removeCollection ends\n');
}/*}}}*/
flockFavoritesService.prototype.updateCollection = function (aCollection, aProperties) {/*{{{*/
    this.setProperties (aCollection, aProperties, FIELD_DESCRIPTION|FIELD_NAME);
}/*}}}*/
flockFavoritesService.prototype.addFavoriteToCollection = function (aFavoriteURL, aCollection) {/*{{{*/
    dump ('addFavoriteToCollection ('+aFavoriteURL+','+rdf_repr(aCollection)+')\n');
    
    // get the favorite resource
    var favorite = RDFS.GetResource (aFavoriteURL);

    // get a container for the collection
    var container = this.Seq (aCollection);

    // if the favorite isn't already in the collection
    if (container.IndexOf (favorite) == -1) {
        // add the favorite to it
        container.AppendElement (favorite);
    }

    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.orderFavoriteInCollection = function (aFavoriteURL, aCollection, aIndex) {/*{{{*/
    dump ('orderFavoriteInCollection ('+aFavoriteURL+','+rdf_repr(aCollection)+', ' + aIndex + ')\n');
    
    // get the favorite resource
    var favorite = RDFS.GetResource (aFavoriteURL);

    // get a container for the collection
    var container = this.Seq (aCollection);

    // if the favorite is already in the collection, remove it
    if (container.IndexOf(favorite) != -1) {
        container.RemoveElement(favorite, true);
        //aIndex = aIndex - 1;
    }
    
    // add the new favorite
    container.InsertElementAt(favorite, aIndex + 1, true);

    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.removeFavoriteFromCollection = function (aFavoriteURL, aCollection) {/*{{{*/
    dump ('removeFavoriteFromCollection ('+aFavoriteURL+','+rdf_repr(aCollection)+')\n');

    // get the favorite resource
    var favorite = RDFS.GetResource (aFavoriteURL);

    // get a container for the collection
    var container = this.Seq (aCollection);

    // remove the favorite from the container
    container.RemoveElement (favorite, true);

    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.setToolbarCollection = function (aCollection) {/*{{{*/
    dump('setToolbarCollection (' + rdf_repr(aCollection) + ')\n');
    var oldCollection = this.dataSource.GetSource(RDF_TOOLBAR, rdfLiteral('true'), true);
    if (oldCollection) {
        this.dataSource.Unassert(oldCollection, RDF_TOOLBAR, rdfLiteral('true'));
    }
 
    this.dataSource.Assert(aCollection, RDF_TOOLBAR, rdfLiteral('true'), true);
    this.dataSource.Flush();
}/*}}}*/
flockFavoritesService.prototype.getToolbarCollection = function () {/*{{{*/
    dump('getToolbarCollection ()\n');
    return this.dataSource.GetSource(RDF_TOOLBAR, rdfLiteral('true'), true);
}/*}}}*/
flockFavoritesService.prototype.addFeed = function (aFavoriteURL, aFeedURL, aFeedTitle) { /*{{{*/
    dump ('addFeed ('+aFavoriteURL+', '+aFeedURL+', "'+aFeedTitle+'")\n');
    // we need to find the favorite and feed
    var resource = RDFS.GetResource (aFavoriteURL);
    var feed = RDFS.GetResource (aFeedURL);

    var favorite = this.Seq(resource);
    
    this.dataSource.Assert(feed, RDF_TYPE, RDF_FEED, true);
    this.dataSource.Assert(feed, RDF_FEEDURL, rdfLiteral(aFeedURL), true);
    
    favorite.AppendElement(feed);
    
    // add the feed to the favorite
    //this.dataSource.Assert (resource, RDF_FEED, feed, true);

    // add a title if any
    if (aFeedTitle) {
        this.dataSource.Assert (feed, RDF_NAME, rdfLiteral (aFeedTitle), true);
    }

    // mark it as disabled
    this.dataSource.Assert (feed, RDF_FEEDENABLED, rdfLiteral ('false'), true);

    
    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.removeFeed = function (aFavoriteURL, aFeedURL) { /*{{{*/
    dump ('removeFeed ('+aFavoriteURL+', '+aFeedURL+')\n');

    // we need to find the favorite and feed
    var resource = RDFS.GetResource (aFavoriteURL);
    var feed = RDFS.GetResource (aFeedURL);

    // remove the "feed" property
    this.dataSource.Unassert (resource, RDF_FEED, feed);

    // FIXME: should we remove other feed properties?

    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.getFeeds = function (aFavoriteURL, count) { /*{{{*/
    // we need to find the favorite
    var resource = RDFS.GetResource (aFavoriteURL);

    // get all flock#feed properties for this favorite
    var arr = new Array ();
    var targets;
    if (RDFCU.IsContainer(this.dataSource, resource)) {
        dump("IT *IS* A CONTAINER!\n");
        var cont = Components.classes[CONTAINER_CONTRACTID].createInstance(nsIRDFContainer);
        cont.Init(this.dataSource, resource);
        targets = cont.GetElements();
    }
    
    //var targets = this.dataSource.GetTargets (resource, RDF_FEED, true);
    while (targets && targets.hasMoreElements ()) {
        // FIXME handle non-resource targets for this
        var item = targets.getNext();
        var itemRsrc = item.QueryInterface(nsIRDFResource);
        arr.push (itemRsrc.ValueUTF8);
    }
    count.value = arr.length;
    return arr;
}/*}}}*/
flockFavoritesService.prototype.setFeedEnabled = function (aFeedURL, aEnabled) { /*{{{*/
    // find the feed
    var feed = RDFS.GetResource (aFeedURL);

    // FIXME: handle missing properties better
    var oldvalue = this.dataSource.GetTarget (feed, RDF_FEEDENABLED, true);
    this.dataSource.Change (feed, RDF_FEEDENABLED, oldvalue, 
            rdfLiteral (aEnabled?"true":"false"));
    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.getFeedEnabled = function (aFeedURL) { /*{{{*/
    // find the feed
    var feed = RDFS.GetResource (aFeedURL);

    // get the property value
    var literal = this.dataSource.GetTarget (feed, RDF_FEEDENABLED, true);
    try {
        return literal.QueryInterface (nsIRDFLiteral).Value == "true";
    } catch (e) {
        return false;
    }
    this.dataSource.Flush ();
}/*}}}*/

flockFavoritesService.prototype.addTag = function (aURL, aTag) { /*{{{*/
    dump ('addTag ('+aURL+', '+aTag+'")\n');
    var str = aTag.replace(/\s/g,"");
    if(str.length==0) return;
    
    var resource = RDFS.GetResource (aURL);
    var tagAlreadyExists = this.dataSource.HasAssertion(resource, RDF_TAG, rdfLiteral(aTag), true);
    if (!tagAlreadyExists) {
        this.dataSource.Assert(resource, RDF_TAG, rdfLiteral(aTag), true);
        this.addFavoriteToTagCollection(aURL, aTag);
    }
    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.removeTag = function (aURL, aTag) { /*{{{*/
    dump ('removeTag ('+aURL+', '+aTag+')\n');
    
    var resource = RDFS.GetResource (aURL);
    var tagAlreadyExists = this.dataSource.HasAssertion(resource, RDF_TAG, rdfLiteral(aTag), true);
    if (tagAlreadyExists) {
        this.dataSource.Unassert(resource, RDF_TAG, rdfLiteral(aTag));
        this.removeFavoriteFromTagCollection(aURL, aTag);
    }
    this.dataSource.Flush ();
}/*}}}*/
flockFavoritesService.prototype.getTags = function (aURL, count) { /*{{{*/
    dump ('getTags ('+aURL+')\n');
    
    var resource = RDFS.GetResource(aURL);

    var arr = new Array();
    var targets = this.dataSource.GetTargets (resource, RDF_TAG, true);
    while (targets && targets.hasMoreElements ()) {
        var item = targets.getNext();
        var itemRsrc = item.QueryInterface(nsIRDFLiteral);
        arr.push(itemRsrc.Value);
    }
    count.value = arr.length;
    return arr;
}/*}}}*/
flockFavoritesService.prototype.setTags = function (aURL, aTags) { /*{{{*/
    dump ('setTags ('+aURL+', '+aTags+')\n');
    
    var resource = RDFS.GetResource(aURL);

    // Clear the old tags
    var oldTags = this.dataSource.GetTargets (resource, RDF_TAG, true);
    while (oldTags && oldTags.hasMoreElements ()) {
        var item = oldTags.getNext();
        var itemRsrc = item.QueryInterface(nsIRDFLiteral);
        this.removeTag(aURL,item.Value);
    }
    
    // Set the new ones
    var newTags = aTags.split(/\s*,\s*/);
    for (var i = 0; i < newTags.length; ++i) {
        this.addTag(aURL,newTags[i]);
    }
    this.dataSource.Flush();
    
}/*}}}*/

flockFavoritesService.prototype.getTagsField = function (aURL) { /*{{{*/
    dump ('getTagsField ('+aURL+')\n');
    var count = {};
    var tags = this.getTags(aURL, count);
    return tags.join(', ');
}/*}}}*/
flockFavoritesService.prototype.updateTagsField = function (aURL) { /*{{{*/
    dump ('updateTagsField ('+aURL+')\n');
    
    var tagsField = this.getTagsField(aURL);
    var resource = RDFS.GetResource(aURL);
    var old_target = this.dataSource.GetTarget (resource, RDF_TAGS, true);
    if (old_target) {
        this.dataSource.Change (resource, RDF_TAGS, old_target, rdfLiteral(tagsField));
    } else {
        this.dataSource.Assert (resource, RDF_TAGS, rdfLiteral(tagField), true);
    }
    this.dataSource.Flush();
}/*}}}*/


flockFavoritesService.prototype.getTagCollection = function (aTag) { /*{{{*/
    dump ('getTagCollection ('+aTag+')\n');
    
    var elements = this.tagsRoot.GetElements();
    while (elements.hasMoreElements()) {
        var item = elements.getNext();
        var itemRsrc = item.QueryInterface(nsIRDFResource);
        if (this.dataSource.HasAssertion(itemRsrc,RDF_NAME, rdfLiteral(aTag), true)) {
            return itemRsrc;
        }        
    }
    return;
}/*}}}*/
flockFavoritesService.prototype.createTagCollection = function (aTag) { /*{{{*/
    dump ('createTagCollection ('+aTag+')\n');
    tagCollection = RDFS.GetAnonymousResource();
    this.dataSource.Assert(tagCollection, RDF_NAME, rdfLiteral(aTag), true);
    this.dataSource.Assert(tagCollection, RDF_TYPE, RDF_COLLECTION, true);
    this.Seq(tagCollection);
    this.tagsRoot.AppendElement(tagCollection);
    return tagCollection;
}/*}}}*/
flockFavoritesService.prototype.removeTagCollection = function (aTag) { /*{{{*/
    dump ('removeTagCollection ('+aTag+')\n');
    
    var tagCollection = this.getTagCollection(aTag);
    if (tagCollection) {
        this.dataSource.Unassert(tagCollection, RDF_TYPE, RDF_COLLECTION);
        this.dataSource.Unassert(tagCollection, RDF_NAME, rdfLiteral(aTag));
        this.tagsRoot.RemoveElement(tagCollection, true);
        this.dataSource.Unassert(tagCollection, RDF_INSTANCEOF, RDF_SEQ);
        // Clear the elements
        tagCollection = this.Seq(tagCollection);
        var elements = tagCollection.GetElements();
        while (elements.hasMoreElements()) {
            container.RemoveElement(elements.getNext(), false);
        }
    }
}/*}}}*/
flockFavoritesService.prototype.addFavoriteToTagCollection = function (aURL, aTag) { /*{{{*/
    dump ('addFavoriteToTagCollection ('+aURL+', '+aTag+')\n');
    
    var favorite = RDFS.GetResource(aURL);
    
    var tagCollection = this.getTagCollection(aTag);
    if (!tagCollection) {
        dump('WASnT THERE!!\n');
        tagCollection = this.createTagCollection(aTag);
    } else {
        dump('WASSS THERE!!\n');
    }
    dump(rdf_repr(tagCollection)+"\n");
    // XXX andy: may want to check for existence first?
    tagCollection = this.Seq(tagCollection);
    tagCollection.AppendElement(favorite, true);
    this.dataSource.Flush();
}/*}}}*/
flockFavoritesService.prototype.removeFavoriteFromTagCollection = function (aURL, aTag) { /*{{{*/
    dump ('removeFavoriteFromTagCollection ('+aURL+', '+aTag+')\n');
    
    var favorite = RDFS.GetResource(aURL);
    
    var tagCollection = this.getTagCollection(aTag);
    if (tagCollection) {
        tagCollection = this.Seq(tagCollection);
        tagCollection.RemoveElement(favorite, true);
        // Test whether children still exist
        var elements = tagCollection.GetElements();
        if (!elements.hasMoreElements()) {
            this.removeTagCollection(aTag);
        }
        this.dataSource.Flush();
    }
}/*}}}*/

flockFavoritesService.prototype.setWebService = function (aName) {/*{{{*/
    this.webService = 
        Components.classes['@flock.com/favorites/webservice;1?name='+aName]
        .getService (flockIWebService);
}/*}}}*/
flockFavoritesService.prototype.getWebServiceEnabled = function () {/*{{{*/
    try {
        return PREFS.getBoolPref (PREF_WEB_SERVICE_ENABLED);
    } catch (e) {
        return false;
    }
}/*}}}*/
flockFavoritesService.prototype.setWebServiceEnabled = function (aEnabled) {/*{{{*/
    PREFS.setBoolPref (PREF_WEB_SERVICE_ENABLED, aEnabled);
    this.webService.setEnabled(aEnabled);
}/*}}}*/
flockFavoritesService.prototype.syncUp = function () {/*{{{*/
    // we should try to synchonize up all un-synced favorites
    dump ('syncUp()\n');
    if (!this.getWebServiceEnabled () || !this.webService) {
        dump ('not syncing\n');
        // if the web service isn't enabled or we don't have a web service 
        // object then lets give up right now
        return;
    }
    var resources = this.dataSource.GetSources (RDF_NSYNC, rdfLiteral ('false'), true);
    while (resources.hasMoreElements ()) {
        var resource = resources.getNext ();
        // coerce resource to being an nsIRDFResource
        resource.QueryInterface (nsIRDFResource);
        if (!resource.ValueUTF8) {
            // badness
            continue;
        }
        if (!this.dataSource.HasAssertion (resource, RDF_TYPE, RDF_FAVORITE, true)) {
            continue;
        }
        var properties = this.getProperties (resource);
        var listener = new Object ();
        listener.resource = resource;
        listener.fsvc = this;
        listener.externalListeners = this.externalListeners;
        listener.onSuccess = function () {
            dump ('web service success\n');
            this.fsvc.setProperties (this.resource, {nsync: true, fields: FIELD_NSYNC});
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncUp(true, "ok");
        }
        listener.onError = function (msg) { 
            dump ('web service error: '+msg+'\n');
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncUp(false, msg);
        }
        listener.onFault = function (msg) { 
            dump ('web service fault: '+msg+'\n');
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncUp(false, msg);
        }
        listener.onStart = function () { 
            for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncUpStart();
        }
        dump ('properties:'+FavoritePropertyMetadata.repr (properties)+'\n');
        dump ('properties.tags:'+properties.tags+'\n');
        this.webService.change (resource.ValueUTF8, properties, listener);
    }
}/*}}}*/
flockFavoritesService.prototype.syncDown = function () {/*{{{*/
    /*
    const WWATCH = Components.classes[WINDOWWATCHER_CONTRACTID].getService (nsIWindowWatcherService);
    var window = WWATCH.openWindow (null, FAVORITES_PROGRESS_URL,
            'favoritesprogress', 'chrome', null);
    dump  ('progress dialog: '+window+'\n');
    */
    var listener = new Object ();
    listener.externalListeners = this.externalListeners;
    listener.onSuccess = function () {
        dump ('web service success\n');
        for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncDown(true, "ok");
    }
    listener.onError = function (msg) { 
        dump ('web service error: '+msg+'\n');
        for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncDown(false, msg);
    }
    listener.onFault = function (msg) { 
        dump ('web service fault: '+msg+'\n');
        for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncDown(false, msg);
    }
    listener.onStart = function () { 
        for(var i=0;i<this.externalListeners.length;++i) this.externalListeners[i].onSyncDownStart();
    }
    this.webService.syncDown (listener);
 }/*}}}*/
flockFavoritesService.prototype.test = function () {/*{{{*/
    dump_rdf (RDFS.GetDataSource('rdf:flock-favorites'))
    //rdf_dot (mem_ds);
}/*}}}*/
flockFavoritesService.prototype.webService = null;

// Module implementation/*{{{*/
var favoritesModule = new Object();

favoritesModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(FAVORITES_CID, 
                                    "Flock Favorites JS Component",
                                    FAVORITES_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
    const CATMAN_CONTRACTID = "@mozilla.org/categorymanager;1";
    const nsICategoryManager = Components.interfaces.nsICategoryManager;
    var catman = Components.classes[CATMAN_CONTRACTID].
                            getService(nsICategoryManager);
                            
    const JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY = "JavaScript global property";
    catman.addCategoryEntry(JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
                            "flockFavorites",
                            FAVORITES_CONTRACTID,
                            true,
                            true);
}

favoritesModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(FAVORITES_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return favoritesServiceFactory;
}

favoritesModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    return true;
}/*}}}*/
    
/* factory object *//*{{{*/
var favoritesServiceFactory = new Object();

favoritesServiceFactory.createInstance =
function (outer, iid) {
    debug("CI: " + iid);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new flockFavoritesService()).QueryInterface(iid);
}/*}}}*/

/* entrypoint *//*{{{*/
function NSGetModule(compMgr, fileSpec) {
    return favoritesModule;
}/*}}}*/

/* static functions *//*{{{*/
if (DEBUG) {
    debug = function (s) { dump("-*- flockFavorites component: " + s + "\n"); }
} else {
    debug = function (s) {}
}/*}}}*/

// string bundle/*{{{*/
var strBundleService = null;
function srGetStrBundle(path)
{
   var strBundle = null;
   if (!strBundleService) {
       try {
          strBundleService =
          Components.classes["@mozilla.org/intl/stringbundle;1"].getService(); 
          strBundleService = 
          strBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);
       } catch (ex) {
          dump("\n--** strBundleService failed: " + ex + "\n");
          return null;
      }
   }
   strBundle = strBundleService.createBundle(path); 
   if (!strBundle) {
       dump("\n--** strBundle createInstance failed **--\n");
   }
   return strBundle;
}/*}}}*/


flockFavoritesService.prototype.externalListeners = null;

flockFavoritesService.prototype.addProgressListener = function (aListener) {
    this.externalListeners[this.externalListeners.length] = aListener;
}

flockFavoritesService.prototype.removeProgressListener = function (aListener) {
    var newListeners = new Array();
    for(var i=0;i<this.externalListeners.length;++i) {
        if(this.externalListeners[i] == aListener) continue;
        newListeners[newListeners.length] = this.externalListeners[i];
    }
    this.externalListeners = newListeners;
}
