//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const FLOCK_RSS_CONTRACTID       = '@flock.com/rssservice;1';
const DEBUG = true; /* set to false to suppress debug messages */
const FLOCK_RSS_CID              = Components.ID('{1572f6fd-0f52-4d73-88b3-a9683b7de9ca}');
const nsISupports                = Components.interfaces.nsISupports;
const nsIFactory                 = Components.interfaces.nsIFactory;
const nsIClassInfo               = Components.interfaces.nsIClassInfo;
const nsITimer                   = Components.interfaces.nsITimer;
const nsIXMLHttpRequest          = Components.interfaces.nsIXMLHttpRequest;
const nsIStandardURL             = Components.interfaces.nsIStandardURL;
const nsIDOMParser               = Components.interfaces.nsIDOMParser;
const nsIDOMSerializer           = Components.interfaces.nsIDOMSerializer;
const flockIHTML2XHTML           = Components.interfaces.flockIHTML2XHTML;
const flockIRSSService           = Components.interfaces.flockIRSSService;
const nsIRDFCompositeDataSource  = Components.interfaces.nsIRDFCompositeDataSource;
const nsIRDFDataSource           = Components.interfaces.nsIRDFDataSource;
const nsIRDFService              = Components.interfaces.nsIRDFService;
const nsIRDFContainerUtils       = Components.interfaces.nsIRDFContainerUtils;
const nsIRDFContainer            = Components.interfaces.nsIRDFContainer;
const nsIRDFNode                 = Components.interfaces.nsIRDFNode;
const nsIRDFResource             = Components.interfaces.nsIRDFResource;
const nsIRDFLiteral              = Components.interfaces.nsIRDFLiteral;
const nsIRDFInt                  = Components.interfaces.nsIRDFInt;
const nsIRDFDate                 = Components.interfaces.nsIRDFDate;
const nsIRDFBlob                 = Components.interfaces.nsIRDFBlob;

const POLL_INTERVAL              = 1000 * 60 * 60;
//const POLL_INTERVAL              = 1000 * 15;

const RSS_RDF_ROOT               = "urn:flock:rss";
const RSS_RDF_NS                 = "http://www.flock.com/rdf#"
const RSS_RDF_FILENAME           = "rss.rdf";

const RSS_FEED_DIR               = "feeds";

const RSS_FEED_FILENAME          = "feed.rdf";
const RSS_CACHE_FILENAME         = "feed.xml";
const RSS_IMAGE_DIR              = "images";

const RSS_RDF_FEED_ROOT          = RSS_RDF_ROOT + ":feeds";
const RSS_RDF_DATE_ROOT          = RSS_RDF_ROOT + ":dates";

const MAXINT                     = 4294967295;

// contract-ids
const COMPOSITE_CONTRACTID       = '@mozilla.org/rdf/datasource;1?name=composite-datasource';
const MEMORY_DS_CONTRACTID       = '@mozilla.org/rdf/datasource;1?name=in-memory-datasource';
const RDFSERVICE_CONTRACTID      = '@mozilla.org/rdf/rdf-service;1';
const RDFCONTUTILS_CONTRACTID    = '@mozilla.org/rdf/container-utils;1';
const CONTAINER_CONTRACTID       = '@mozilla.org/rdf/container;1';
const STANDARDURL_CONTRACTID     = '@mozilla.org/network/standard-url;1';
const DOMSERIALIZER_CONTRACTID   = '@mozilla.org/xmlextras/xmlserializer;1';

// rdf namespaces
function Namespace (ns) { return function (arg) { return RDFS.GetResource (ns+arg); } }
const W3RDF                      = Namespace ('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
const NSRDF                      = Namespace ('http://home.netscape.com/NC-rdf#');
const FLRDF                      = Namespace ('http://www.flock.com/rdf#');

const RDFS = Components.classes[RDFSERVICE_CONTRACTID].getService (nsIRDFService);
const RDFCU = Components.classes[RDFCONTUTILS_CONTRACTID].getService (nsIRDFContainerUtils);

// dump rdf
function dump_rdf (data_source) {
    var ser=Components.classes["@mozilla.org/rdf/xml-serializer;1"].createInstance(Components.interfaces.nsIRDFXMLSerializer);
    ser.init(data_source);
    ser.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(
    {
        close : function(){dump("\n");},
        flush : function(){},
        write : function (buffer,count){ dump (buffer); return count; },
        writeFrom : function (stream,count){},
        isNonBlocking: false
    });
}

function get_container (ds, node) {
    if (!RDFCU.IsContainer (ds, node)) {
        return null;
    }
    var container = Components.classes[CONTAINER_CONTRACTID].createInstance (nsIRDFContainer);
    container.Init (ds, node);
    return container;
}


function dd(obj) {
    for (p in obj) {
        try {
            dump(p + ' :: ' + obj[p] + '\n'); 
            if (typeof obj[p] == 'object') { 
                for (q in obj[p]) { 
                    dump(p + '.' + q + ' :: ' + obj[p][q] + '\n'); 
                } 
            }
        } catch(e) {
            continue;
        }
    }
}


// Service implementation
function flockRSSService()
{
    debug("Constructed\n");

    this.init() 

    const TIMER_CONTRACTID = "@mozilla.org/timer;1";
    this.timer =
        Components.classes[TIMER_CONTRACTID].getService(nsITimer);
    this.timer.initWithCallback(this, POLL_INTERVAL, 1);
}


flockRSSService.prototype.notify = function (obj) {
    this.refreshAllFeeds(null);
}

flockRSSService.prototype.test = function () {
    dump("hi rss!\n");
    return;

    //this.refreshAllFeeds(null);
    var a = new Array();
    a[0] = 'http://www.schneier.com/blog/atom.xml';
    //a[1] = 'http://xml.metafilter.com/atom.xml';

    this.refreshFeed(a[0]);
    //this.refreshFeed(a[1]);
    var ds = this.getAggregateFeeds('dates', a.length, a);
    return ds;
}

flockRSSService.prototype.init = function () {
    var rssFile = Components.classes["@mozilla.org/file/directory_service;1"].
        getService(Components.interfaces.nsIProperties).
        get("ProfD", Components.interfaces.nsIFile);

    rssFile.append(RSS_RDF_FILENAME);
    if (!rssFile.exists()) {
        rssFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
    }

    var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);
                              
    var fileHandler = ioSvc.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var furl = fileHandler.getURLSpecFromFile(rssFile);

    eval(flock_JSInclude("rdfds.js.lib"));
    this.ds = new RDFDataSource(furl);

    this.root = this.ds.getNode(RSS_RDF_ROOT);
    this.root.makeSeq();
    this.ds.save();

    var rssDir = Components.classes["@mozilla.org/file/directory_service;1"].
        getService(Components.interfaces.nsIProperties).
        get("ProfD", Components.interfaces.nsIFile);
   
    rssDir.append(RSS_FEED_DIR); 
    if (!rssDir.exists()) {
        rssDir.createUnique(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
    }
}

// the nsIClassInfo implementation
flockRSSService.prototype.flags = nsIClassInfo.Singleton;
flockRSSService.prototype.classDescription = "Flock RSS Service";
flockRSSService.prototype.getInterfaces = function (count) {
    var interfaceList = [flockIRSSService, nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
}
flockRSSService.prototype.getHelperForLanguage = function (count) {return null;}

// the nsISupports implementation
flockRSSService.prototype.QueryInterface =
function (iid) {
    if (!iid.equals(flockIRSSService) && 
	!iid.equals(nsITimer) &&
	!iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
}

// Module implementation
var rssModule = new Object();

rssModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(FLOCK_RSS_CID, 
                                    "Flock RSS JS Component",
                                    FLOCK_RSS_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
    const CATMAN_CONTRACTID = "@mozilla.org/categorymanager;1";
    const nsICategoryManager = Components.interfaces.nsICategoryManager;
    var catman = Components.classes[CATMAN_CONTRACTID].
                            getService(nsICategoryManager);
                            
    const JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY = "JavaScript global property";
    catman.addCategoryEntry(JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
                            "flockRSS",
                            FLOCK_RSS_CONTRACTID,
                            true,
                            true);
}

rssModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(FLOCK_RSS_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return rssServiceFactory;
}

rssModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    return true;
}
    
/* factory object */
var rssServiceFactory = new Object();

rssServiceFactory.createInstance =
function (outer, iid) {
    debug("CI: " + iid);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new flockRSSService()).QueryInterface(iid);
}

/* entrypoint */
function NSGetModule(compMgr, fileSpec) {
    return rssModule;
}

/* static functions */
if (DEBUG) {
    debug = function (s) { dump("-*- flockRSS component: " + s + "\n"); }
} else {
    debug = function (s) {}
}

var strBundleService = null;
function srGetStrBundle(path)
{
   var strBundle = null;
   if (!strBundleService) {
       try {
          strBundleService =
          Components.classes["@mozilla.org/intl/stringbundle;1"].getService(); 
          strBundleService = 
          strBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);
       } catch (ex) {
          dump("\n--** strBundleService failed: " + ex + "\n");
          return null;
      }
   }
   strBundle = strBundleService.createBundle(path); 
   if (!strBundle) {
       dump("\n--** strBundle createInstance failed **--\n");
   }
   return strBundle;
}

flockRSSService.prototype.hasFeed =
function (url)
{
    var node = this.ds.getNode(url);
    return node.targetExists(RSS_RDF_NS + 'loaded', 'true');
}

flockRSSService.prototype.addFeed =
function (url)
{
    var node = this.ds.getNode(url);
    this.root.addChild(node, false);
    this.ds.save();

    this.refreshFeed(url)
}

flockRSSService.prototype.removeFeed =
function (url)
{
    var node = this.ds.getNode(url);
    this.root.removeChild(node);
    this.ds.save();

    var rssDir = getFeedDir(url);

    if (rssDir.exists())
        rssDir.remove(true);
}

flockRSSService.prototype.getFeed =
function (url)
{
    return getFeedInternal(url);
}

flockRSSService.feed_queue = new Object();

flockRSSService.prototype.refreshFeed =
function (url)
{
    var ds = this.ds;
    const XMLHTTPREQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID].createInstance(nsIXMLHttpRequest);
    req.open('GET', url, true);
    req.onreadystatechange = function (aEvt) {
        if (req.readyState == 4) {
            if (req.status == 200)
                TryParseRSS(ds, url, req.responseText);
            else
                dump("Error loading " + url + "\n");
        }
    };
    req.send(null); 
}

flockRSSService.prototype.refreshAllFeeds =
function (listener)
{
    var urls = this.root.getChildren();
    var total = this.root.getChildCount();

    if (total > 0)
        refreshFeeds(this.ds, urls, 0, total, listener);
}

flockRSSService.prototype.isFeed =
function (dom)
{
    return this.getFeedType(dom) ? true : false;
}

flockRSSService.prototype.getFeedType =
function (dom)
{
    return SniffFeedType(dom);
}

function flock_JSInclude(filename)
{
    // get profile directory
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ComsD", Components.interfaces.nsIFile);

    file.append(filename);

    var data = "";
    var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                           .createInstance(Components.interfaces.nsIFileInputStream);
    var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"]
                           .createInstance(Components.interfaces.nsIScriptableInputStream);

    fstream.init(file, 1, 0, false);
    sstream.init(fstream); 

    var str = sstream.read(-1);
    while (str.length > 0) {
        data += str;
        str = sstream.read(-1);
    }

    sstream.close();
    fstream.close();

    return data;
}

function escape_url(url)
{
    var rval = url.replace(/\/+$/, '');
    rval = rval.replace(/[\?\/\\:.]+/g, '_');
    return escape(rval);
}

function getFeedDir(url)
{
    var rssDir = Components.classes["@mozilla.org/file/directory_service;1"].
        getService(Components.interfaces.nsIProperties).
        get("ProfD", Components.interfaces.nsIFile);
    rssDir.append(RSS_FEED_DIR);
    rssDir.append(escape_url(url));

    return rssDir;
}

function getFeedFile(url)
{
    var rssFile = getFeedDir(url);
    rssFile.append(RSS_FEED_FILENAME);
    return rssFile;
}

function getFeedCacheFile(url)
{
    var rssFile = getFeedDir(url);
    rssFile.append(RSS_CACHE_FILENAME);
    return rssFile;
}

function getFeedImageDir(url)
{
    var rssFile = getFeedDir(url);
    rssFile.append(RSS_IMAGE_DIR);
    return rssFile;
}

function getFeedInternal(url)
{
    var rssFile = getFeedFile(url);

    if (!rssFile.exists())
        return null;

    var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);

    var fileHandler = ioSvc.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var furl = fileHandler.getURLSpecFromFile(rssFile);

    eval(flock_JSInclude("rdfds.js.lib"));
    var ds = new RDFDataSource(furl);

    //dump(ds.serializeToString());
    return ds.getRawDataSource();
}

function saveData(file, data)
{
    if (file.exists())
        file.remove(false);

    file.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);

    var ostream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                           .createInstance(Components.interfaces.nsIFileOutputStream);
    ostream.QueryInterface(Components.interfaces.nsIOutputStream);
    ostream.QueryInterface(Components.interfaces.nsISeekableStream);
    ostream.init(file, 0x04 | 0x08, 420, 0);
    ostream.write(data, data.length);
    ostream.flush();
    ostream.close();
}

function SniffFeedType(dom)
{
    var nodes = dom.getElementsByTagName('feed');
    if (nodes.length > 0) {
        var node = nodes[0];
        var ns = node.getAttribute('xmlns');

        if (ns == 'http://www.w3.org/2005/Atom')
            return 'Atom 1.0';
        else
            return 'Atom 0.3';
    }

    nodes = dom.getElementsByTagName('RDF');
    if (nodes.length > 0) {
        var node = nodes[0];

        if (dom.getElementsByTagName('channel').length == 0)
            return null;

        var ns = node.getAttribute('xmlns');
        if (ns == 'http://purl.org/rss/1.0/')
            return 'RSS 1.0';
        else
            return 'RSS 0.9';
    }

    nodes = dom.getElementsByTagName('rss');
    if (nodes.length > 0) {
        var node = nodes[0];

        if (dom.getElementsByTagName('channel').length == 0)
            return null;

        var version = node.getAttribute('version');

        if (version)
            return 'RSS ' + version;
        else
            return 'RSS';
    }

    return null;
}

function TryParseRSS(global_ds, url, feed_data)
{
    dump("Parsing " + url + "\n");

    var cacheFile = getFeedCacheFile(url);
    saveData(cacheFile, feed_data);
 
    var rssFile = getFeedFile(url);

    if (!rssFile.exists()) {
        rssFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
    }

    var ioSvc = Components.classes["@mozilla.org/network/io-service;1"]
                              .getService(Components.interfaces.nsIIOService);

    var fileHandler = ioSvc.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var furl = fileHandler.getURLSpecFromFile(rssFile);

    eval(flock_JSInclude("rdfds.js.lib"));
    var ds = new RDFDataSource(furl);

    var dateroot = ds.getNode(RSS_RDF_DATE_ROOT);
    dateroot.makeBag();

    var feedroot = ds.getNode(RSS_RDF_FEED_ROOT);
    feedroot.makeBag();

    var root = ds.getNode(url);
    root.makeBag();

    feedroot.addChild(root);

    const DOMPARSER_CONTRACTID = "@mozilla.org/xmlextras/domparser;1";
    var parser = Components.classes[DOMPARSER_CONTRACTID].createInstance(nsIDOMParser);
    var isAtom = false;

    var dom = parser.parseFromString(feed_data, "text/xml");

    var feedtype = SniffFeedType(dom);
    if (!feedtype)
        feedtype = "Unknown";

    root.addTargetOnce(RSS_RDF_NS + "feedtype", feedtype);
    
    var feeds = dom.getElementsByTagName('feed');

    var channel = null;
    if (feeds.length > 0) {
        channel = feeds[0];
        isAtom = true;
    } else {
        var channels = dom.getElementsByTagName('channel');
        if (channels.length > 0) {
            channel = channels[0];
        }
    }

    if (channel == null) {
        dump("Error parsing " + url + "\n");
        return;
    }

    var feed = {};
    for (var i = 0; i < channel.childNodes.length; i++) {
        var n = channel.childNodes[i];
        if (n.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
            switch (n.nodeName) {
                case "title":
                case "subtitle":
                case "link":
                case "description":
                case "tagline":
                case "dc:date":
                case "pubDate":
                case "author":
                //case "ttl":
                //case "id":
                    var name = n.nodeName;
                    var node_data;

                    if (name == "tagline" || name == "subtitle")
                        name = "description";
                    else if (name == "managingEditor")
                        name = "author";

                    if (name == "description")
                        node_data = getHTMLContent(n);
                    else if (isAtom && name == "author")
                        node_data = getAuthor(n);
                    else if (name == "link")
                        node_data = getLinkURL(n);
                    else
                        node_data = getNodeText(n);

                    if (name == "dc:date" || name == "pubDate" ||
                        name == "updated" || name == "modified") {
                        name = "date";
                        node_data = parseDate(node_data);
                    }

                    //dump(name + ': ' + node_data + '\n');
                    if (node_data)
                        feed[name] = node_data;

                    break;
            }
        }
    }

    if (!feed.title)
        feed.title = url;

    for (var f in feed) {
        root.addTargetOnce(RSS_RDF_NS + f, feed[f]);
    }
    
    now = new Date();
    root.addTargetOnce(RSS_RDF_NS + "last_refreshed", now.toString());

    var images = dom.getElementsByTagName('image');
    if (images.length > 0) {
        var image = images[0];

        for (var i = 0; i < image.childNodes.length; i++) {
            var c = image.childNodes[i];
            if (c.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
                switch (c.nodeName) {
                    case "url":
                        root.addTargetOnce(RSS_RDF_NS + 'image', getNodeText(c));
                        break;
                }
            }
        }
    }

    var items;
    if (isAtom) {
        items = dom.getElementsByTagName('entry'); 
    } else {
        items = dom.getElementsByTagName('item'); 
    }

    var authors = {};
    for (var i = 0; i < items.length; i++) {
        var n = items[i];
        var post = {};

        for (var j = 0; j < n.childNodes.length; j++) {
            var c = n.childNodes[j];
            if (c.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
                switch (c.nodeName) {
                    case "title":
                    case "author":
                    case "link":
                    case "description":
                    case "content:encoded":
                    case "content":
                    case "summary":
                    case "dc:date":
                    case "pubDate":
                    case "modified":
                    case "updated":
                    case "issued":
                    case "published":
                    case "guid":
                    case "id":
                        var name = c.nodeName;
                        var node_data;

                        if (name == 'content:encoded')
                            name = "content";

                        if (name == "content" || name == "description") {
                            if (isAtom)
                                node_data = getAtomContent(c);
                            else
                                node_data = getHTMLContent(c);
                        } else if (isAtom && name == "author")
                            node_data = getAuthor(c);
                        else if (name == "link")
                            node_data = getLinkURL(c);
                        else
                            node_data = getNodeText(c);

                        if (name == "dc:date" || name == "pubDate" ||
                            name == "issued" || name == "published") {
                            name = "date";
                            node_data = parseDate(node_data);
                        } else if (name == "updated" || name == "modified") {
                            name = "updated";
                            node_data = parseDate(node_data);
                        }

                        if (node_data)
                            post[name] = node_data;

                        break;
                }
            }
        }

        var post_id = null;
        if (post.id)
            post_id = post.id;
        else if (post.guid)
            post_id = post.guid;
        else if (post.link)
            post_id = post.link;
        else if (post.date)
            post_id = post.date;
        else if (post.title)
            post_id = post.title;

        var post_desc = null;
        if (post.description)
            post_desc = post.description;
        else if (post.content)
            post_desc = post.content;
        else if (post.summary)
            post_desc = post.summary;
        else if (post.link)
            post_desc = post.link;

        if (!post.date && post.updated)
            post.date = post.updated;

        post.description = post_desc + "\n";
      
        if (feed.link) 
            post.feedurl = feed.link;

        post.feedtitle = feed.title;

        var date;
        if (post.date) {
            date = new Date(post.date);
        } else {
            date = new Date();
        }

        if (post.author)
            authors[post.author] = 1;
        else
            authors[''] = 1;

        post.datevalue = getDateValue(date);

        var timeval = date.toLocaleFormat('%I:%M') +
                      date.toLocaleFormat('%p').toLowerCase();
        post.time = timeval.replace(/^0/, '');

        var date_only = new Date(date.toDateString());
        var date_repr = date_only.toLocaleFormat('%b %d');
        date_repr = date_repr.replace(' 0', ' ');

        post.shortdate = date_repr;
         
        var pnode;
        if (post_id)
            pnode = ds.getNode(RSS_RDF_ROOT + ":post:" + post_id);
        else
            pnode = ds.getAnonymousNode();

        for (var p in post) {
            pnode.addTargetOnce(RSS_RDF_NS + p, post[p]);
        }

        root.addChild(pnode);

        date_repr = date_only.toLocaleFormat('%A, %B %d, %Y');
        date_repr = date_repr.replace(' 0', ' ');

        var dateNode = ds.getNode(RSS_RDF_DATE_ROOT + ':' + date_only.getTime());
        dateNode.addTargetOnce(RSS_RDF_NS + 'title', date_repr);
        dateNode.addTargetOnce(RSS_RDF_NS + 'datevalue', getDateValue(date_only));

        dateroot.addChild(dateNode);

        dateNode.makeBag();
        dateNode.addChild(pnode);
    }

    if (isAtom && !feed.author) {
        var authorCount = 0;
        var author1;
        for (var a in authors) {
            authorCount++;
            if (authorCount > 1)
                break;
            author1 = a;
        }

        if (authorCount == 1 && author1) {
            root.addTargetOnce(RSS_RDF_NS + "author", author1);
        }
    }

    if (global_ds) {
        var unode = global_ds.getNode(url);
        unode.addTargetOnce(RSS_RDF_NS + 'loaded', 'true');
        global_ds.save();
    } 
}

function getNodeText(node)
{
    var text = "";
    for (var i = 0; i < node.childNodes.length; i++) {
        var n = node.childNodes[i];
        if (n.nodeType == Components.interfaces.nsIDOMNode.TEXT_NODE ||
            n.nodeType == Components.interfaces.nsIDOMNode.CDATA_SECTION_NODE) {
            text += n.nodeValue;
        }
    }
    return text;
} 

function getLinkURL(node)
{
    if (node.hasAttribute("href")) {
        if (!node.hasAttribute("rel") || node.getAttribute("rel") == "alternate")
            return node.getAttribute("href");
    } else
        return getNodeText(node);
}

function getHTMLContent(node)
{
    const HTML2XHTML_CONTRACTID = "@flock.com/html2xhtml;1";
    var html2xhtml = Components.classes[HTML2XHTML_CONTRACTID].createInstance(flockIHTML2XHTML);

    return html2xhtml.convert(getNodeText(node));
}

function getAuthor(node)
{
    for (var i = 0; i < node.childNodes.length; i++) {
        var n = node.childNodes[i];
        if (n.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
            if (n.nodeName == "name")
                return getNodeText(n);
        }
    }

    return null;
}

function getAtomContent(node)
{
    if (node.hasAttribute("type") &&
        (node.getAttribute("type") == "xhtml" ||
         node.getAttribute("type") == "application/xhtml+xml")) {
        var serializer = Components.classes[DOMSERIALIZER_CONTRACTID].createInstance(nsIDOMSerializer);

        for (var i = 0; i < node.childNodes.length; i++) {
            var n = node.childNodes[i];
            if (n.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
                return serializer.serializeToString(n);
            }
        }
    }
    else if (node.hasAttribute("type") &&
             (node.getAttribute("type") == "html" ||
              node.getAttribute("type") == "text/html")) {
        return getHTMLContent(node);
    }

    return getNodeText(node);
}

function refreshFeeds(ds, urls, current, total, listener)
{
    if (!urls.hasMoreElements()) {
        dump("RSSService: done refreshing feeds\n");
        if (listener)
            listener.onProgress(current, total, 'Done');
        return;
    }

    var urlNode = urls.getNext();
    var url = urlNode.getValue();
    //dump("Getting " + url + "\n");

    if (listener)
        listener.onProgress(current, total, 'Loading ' + url);

    const XMLHTTPREQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID].createInstance(nsIXMLHttpRequest);
    try
    {
        req.open('GET', url, true);
    }
    catch(e)
    {
        //likely a url error;
        return;
    }
    req.onreadystatechange = function (aEvt) {
        if (req.readyState == 4) {
            if (req.status == 200)
                TryParseRSS(ds, url, req.responseText);
            else
                dump("Error loading " + url + "\n");

            refreshFeeds(ds, urls, current + 1, total, listener);
        }
    };
    req.send(null);
}

flockRSSService.prototype.getAggregateFeeds =
function (sortkey, len, urls)
{
    eval(flock_JSInclude("rdfds.js.lib"));
    var memds = new RDFDataSource();

    /*
    var memroot = memds.getNode(RSS_RDF_ROOT + ':' + sortkey);
    memroot.makeBag(); 
    */

    for (i = 0; i < urls.length; i++) {
        var feed = urls[i];
        var feed_ds = this.getFeed(feed);

        var feed_helper = new RDFDataSource();
        feed_helper.Init(feed_ds);

        feed_helper.copyAllToDataSource(memds);

/*
        if (sortkey == 'feeds') {
            var urlnode = memds.getNode(feed);
            memroot.addChild(urlnode);
        } else if (sortkey == 'dates') {
            var root = memds.getNode(feed);
            var posts = root.getChildren();

            while (posts && posts.hasMoreElements()) {
                var post = posts.getNext();

                var dateObj = post.getTarget(RSS_RDF_NS + 'datevalue');
                if (dateObj) {
                    var date = getDateFromValue(dateObj.getValue());
                    var dateStr = date.toDateString();

                    var dateNode = memds.getNode(RSS_RDF_ROOT + ":" + dateStr);

                    dateNode.addTargetOnce(RSS_RDF_NS + 'title', dateStr);

                    var dateNum = getDateValue(new Date(dateStr));
                    dateNode.addTargetOnce(RSS_RDF_NS + 'datevalue', dateNum);

                    memroot.addChild(dateNode);
                    dateNode.makeSeq();
                    dateNode.addChild(post);
                }
            }
        } else {
            dump("Bad sort key: " + sortkey + "\n");
        }
*/
    }

    //dump(memds.serializeToString());
    return memds.getRawDataSource();
}

function getDateValue(date)
{
   const MAXLEN = String(MAXINT).length;

   var val = MAXINT - date.getTime() / 1000;
   var str = String(val);
   
   var numpad = MAXLEN - str.length;

   var zeros = "";
   for (var i = 0; i < numpad; i++) {
       zeros += "0";
   }

   return zeros + str;
}

function getDateFromValue(datevalue)
{
    var val = Number(datevalue);
    return new Date((MAXINT - val) * 1000);
}

function parseDate(datestr)
{
    var date = null;

    datestr = datestr.replace(/^\s*|\s*$/g, '');

    var dateval = Date.parse(datestr);

    if (dateval) {
        date = new Date(dateval);
    } else {
        /* this is a parser for the W3C subset of ISO 8601 dates
         * http://www.w3.org/TR/NOTE-datetime
         */
        const re_iso8601 = new RegExp(
            "^(\\d{4})(-(\\d{2,3})(-(\\d{2})" +
            "(T(\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?" +
            "(Z|(([-+])(\\d{2}):(\\d{2})))?)?)?)?$");

        var m = datestr.match(re_iso8601);

        if (m) {
            date = new Date(Date.UTC(m[1]))

            m[3]  && date.setUTCMonth   (m[3] - 1);
            m[5]  && date.setUTCDate    (m[5]);
            m[7]  && date.setUTCHours   (m[7]);
            m[8]  && date.setUTCMinutes (m[8]);
            m[10] && date.setUTCSeconds (m[10]);

            m[12] && date.setUTCMilliseconds(('0.' + m[12]) * 1000);

            if (m[14]) {
                var tz_off = m[16] * 60 + Number(m[17]);
                tz_off *= m[15] == '-' ? -1 : 1;
                tz_off *= 60 * 1000;

                date.setTime(date.getTime() - tz_off);
            }

        }
    }

    if (date)
        return date.toString();
    else
        return null;
}

flockRSSService.prototype.prioritizeFeeds =
function (len, urls, listener)
{
    getPriorityFeeds(urls, 0, donePriorityFeeds, listener);
}

function donePriorityFeeds(urls, listener)
{
    eval(flock_JSInclude("rdfds.js.lib"));

    var feeds = {};

    for (var u = 0; u < urls.length; u++) {
        var url = urls[u];
        var feed = getFeedInternal(url);

        if (!feed) {
            dump("RSS Error: " + url + " doesn't have a feed!\n");
            continue;
        }

        var ds = new RDFDataSource();
        ds.Init(feed);

        var feedNode = ds.getNode(url);
        var title = feedNode.getTarget(RSS_RDF_NS + 'title').getValue();

        if (!feeds[title]) {
            feeds[title] = new Array();
        }

        var feedinfo = {};
        feedinfo.type = feedNode.getTarget(RSS_RDF_NS + 'feedtype').getValue();
        feedinfo.url = url;

        feeds[title].push(feedinfo);
    }
 
    var priorities = {
       "Atom 1.0": 100,
       "Atom 0.3": 75,
       "RSS 2.0": 50,
       "RSS 0.92": 40
    };

    function sorter(a,b) {
        var t1 = priorities[a.type];
        var t2 = priorities[b.type];

        if (!t1) t1 = 0;
        if (!t2) t2 = 0;

        return t2 - t1;
    }

    var returls = new Array();

    for (var t in feeds) {
         feeds[t].sort(sorter);
         returls.push(feeds[t][0].url);
    }

    listener.onComplete(returls.length, returls);
}

function getPriorityFeeds(urls, index, internalComplete, externalComplete)
{
    if (index >= urls.length) {
        internalComplete(urls, externalComplete);
    }

    var url = urls[index];
    //dump("Getting " + url + "\n");

    const XMLHTTPREQUEST_CONTRACTID = "@mozilla.org/xmlextras/xmlhttprequest;1";
    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID].createInstance(nsIXMLHttpRequest);
    try
    {
        req.open('GET', url, true);
    }
    catch(e)
    {
        //likely a url error;
        return;
    }
    req.onreadystatechange = function (aEvt) {
        if (req.readyState == 4) {
            if (req.status == 200)
                TryParseRSS(null, url, req.responseText);
            else
                dump("Error loading " + url + "\n");

            getPriorityFeeds(urls, index + 1, internalComplete, externalComplete);
        }
    };
    req.send(null);
}

/*
 * TODO
 *
 * handle ttl, hour stuff
 */
