// simple del.icio.us client
// vim: cindent
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//


// FIXME:
// - if multiple changes to a single favorite are queued then those changes 
//   should be merged.

const MIN_WAIT = 1; // wait a minimum of two seconds between calls
const TIMEOUT = 10; // cancel requests that take more than 10 seconds

// seconds since epoch
function secondsSinceEpoch () { return (new Date()).getTime()/1000; }

const re_iso8601 = new RegExp(
    "^(\\d{4})(-(\\d{2})(-(\\d{2})" +
    "(T(\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?" +
    "(Z|(([-+])(\\d{2}):(\\d{2})))?)?)?)?$");

function parseDate(datestr)
{
    var date = null;

    datestr = datestr.replace(/^\s*|\s*$/g, '');

    var dateval = Date.parse(datestr);

    if (dateval) {
        date = new Date(dateval);
    } else {
        /* this is a parser for the W3C subset of ISO 8601 dates
         * http://www.w3.org/TR/NOTE-datetime
         */
        var m = datestr.match(re_iso8601);

        if (m) {
            date = new Date(Date.UTC(m[1]))

            m[3]  && date.setUTCMonth   (m[3] - 1);
            m[5]  && date.setUTCDate    (m[5]);
            m[7]  && date.setUTCHours   (m[7]);
            m[8]  && date.setUTCMinutes (m[8]);
            m[10] && date.setUTCSeconds (m[10]);

            m[12] && date.setUTCMilliseconds(('0.' + m[12]) * 1000);

            if (m[14]) {
                var tz_off = m[16] * 60 + Number(m[17]);
                tz_off *= m[15] == '-' ? -1 : 1;
                tz_off *= 60 * 1000;

                date.setTime(date.getTime() - tz_off);
            }

        }
    }

    return date;
}


// the DeliciousCall object wraps up everything involved in a call
function DeliciousCall (noun, verb, args) {
    dump ('DelicousCall ('+noun+','+verb+','+args+')\n');
    var url = noun+'/'+verb;
    var argsarray = new Array ();
    if (!args) {
        args = new Array ();
    }
    for (key in args) {
        dump (' '+key+'->'+args[key]+'\n');
        argsarray.push (encodeURIComponent(key) + '=' + 
                encodeURIComponent(args[key]));
    }
    if (argsarray.length) {
        url = url + '?' + argsarray.join('&');
    }
    dump ('url: '+url+'\n');
    this.url = url;
    this.request = null; // will be set to the XMLHttpRequest when its sent
}
DeliciousCall.prototype.onError = function (message) {
    dump ('DeliciousCall.onError ('+message+')\n');
}

DeliciousCall.prototype.onResult = function (message, xml) {
    dump ('DeliciousCall.onResult ('+message+','+xml+')\n');
}
DeliciousCall.prototype.onTimeout = function () {
    dump ('DeliciousCall.onTimeout ()\n');
}
DeliciousCall.prototype.requestComplete = function () {
    dump ('DeliciousCall.requestComplete\n');
    try {
        var text = this.request.responseText;
        // strip invalid XML control characters from del.icio.us response
        // the only allowed control chars are \x09 \x0A \x0D
        text = text.replace (/[\x00\x01\x02\x03\x04\x05\x06\x07\x08\x0B\x0C\x0E\x0F\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F]/g, '');
        var xml = Components.classes['@mozilla.org/xmlextras/domparser;1']
                .createInstance(Components.interfaces.nsIDOMParser)
                .parseFromString (text, 'text/xml');
        dump (' xml='+xml+'\n');
        dump (' this.request.responseText='+text+'\n');
        if (this.request.status == 200) {
            this.onResult (this.request.responseText, xml);
        } else {
            this.onError (this.request.status + "");
        }
    } catch (e) {
        this.onError ('exception '+e);
    }
}



// the Delicious object manages a "connection" to a DelicousAPI server.
// it handles queuing and throttling of requests and authentication
function Delicious (url) {
    if (url) {
        this.url = url;
    } else {
        this.url = 'http://del.icio.us/api/';
    }

    this.queue = new Array ();
    this.inProgress = false;
    this.inHTTP = false;
    this.lastCall = null;
    this.lastCallStarted = 0;
    this.lastCallEnded = 0;
    this.enabled = true;

    // set up a 1 second repeating timer
    this.timer = Components.classes["@mozilla.org/timer;1"]
        .createInstance(Components.interfaces.nsITimer);
    this.timer.initWithCallback (this, 1000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
}

Delicious.prototype.setEnabled = function (aEnabled) {
    this.enabled = aEnabled;
    if(!aEnabled) {
        this.queue = new Array();
    }
}

// nsITimer callback
Delicious.prototype.notify = function (timer) {
    // dump ('timer callback\n');
    // dump (' inProgress = '+this.inProgress+'\n');
    // dump (' queue.length = '+this.queue.length+'\n');
    // dump (' lastCallEnded = '+this.lastCallEnded+'\n');
    // if we can send another request lets do it
    if (this.canSendRequest ()) {
        dump ('sending next request');
        this.sendNextRequest ();
        return;
    }

/* FIXME: timeout support in br0ken
    // if the current request has timed out 
    if (this.inHTTP && this.lastCallStarted 
            && (this.lastCallStarted+TIMEOUT > secondsSinceEpoch())) {
        // lets cancel it
        dump ('timed out after '+(secondsSinceEpoch()-this.lastCallStarted)+' seconds (timeout: '+TIMEOUT+'s)\n');
        dump (' '+(this.lastCallStarted+TIMEOUT)+' > '+secondsSinceEpoch()+'\n');

        this.lastCall.request.abort ();

        dump ('calling onTimeout()\n');
        // pass an error back
        if (this.lastCall) {
            this.lastCall.onTimeout ()
        }

        // and get ready for the next call
        this.lastCall = null;
        this.lastCallEnded = secondsSinceEpoch ();
        this.inProgress = false;
        dump ('\n\n\n\n\n\n\n');
        return;
    }
*/
}


Delicious.prototype.submit = function (call) {
    dump ('Delicious.submit ('+call+')\n');
    // add this call to the queue
    this.queue.push (call);
}


Delicious.prototype.canSendRequest = function () {
    return !this.inProgress 
        && (this.queue.length > 0)
        && this.enabled
        && (this.lastCallEnded+MIN_WAIT<=secondsSinceEpoch());
}


Delicious.prototype.sendNextRequest = function () {
    dump ('Delicious.sendNextRequest\n');
    // now we have a call in progress
    this.inProgress = true;
    this.lastCallStarted = secondsSinceEpoch ();

    // get the call
    this.lastCall = this.queue.shift ();

    // build the request
    this.inHTTP = true;
    this.lastCall.request = 
        Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"]
        .createInstance (Components.interfaces.nsIXMLHttpRequest);
    dump ('request url: '+(this.url+this.lastCall.url)+'\n');
    this.lastCall.request.open ('GET', this.url+this.lastCall.url, true);
    var delicious = this;
    this.lastCall.request.onreadystatechange = function (aEvt) {
        dump ('onReadyStateChange: event='+aEvt+' readyState='+delicious.lastCall.request.readyState+'\n');
        switch (delicious.lastCall.request.readyState) {
            case 0:
                dump (' uninitialized\n');
                break;
            case 1:
                dump (' open called\n');
                break;
            case 2:
                dump (' send called\n');
                break;
            case 3:
                dump (' downloading\n');
                break;
            case 4:
                dump (' complete!\n');
                delicious.inHTTP = false;
                // FIXME: handle 503
                // call the callbacks
                delicious.lastCall.requestComplete ();
                // and clean up
                delicious.lastCall = null;
                delicious.lastCallEnded = secondsSinceEpoch ();
                delicious.inProgress = false;
                dump ('\n\n\n\n\n\n\n');
                break;
        }
    };
    // FIXME: change user agent?
    this.lastCall.listener.onStart();
    this.lastCall.request.send (null);
}



// now - the XPCOM service
const flockIFavoriteProperties   = Components.interfaces.flockIFavoriteProperties;
const FIELD_WSID                 = flockIFavoriteProperties.FIELD_WSID;
const FIELD_TAGS                 = flockIFavoriteProperties.FIELD_TAGS;
const FIELD_PERMISSIONS          = flockIFavoriteProperties.FIELD_PERMISSIONS;
const FIELD_NAME                 = flockIFavoriteProperties.FIELD_NAME;
const FIELD_SHORTCUTURL          = flockIFavoriteProperties.FIELD_SHORTCUTURL;
const FIELD_DESCRIPTION          = flockIFavoriteProperties.FIELD_DESCRIPTION;
const FIELD_NSYNC                = flockIFavoriteProperties.FIELD_NSYNC;
const FIELD_LASTVISITDATE        = flockIFavoriteProperties.FIELD_LASTVISITDATE;
const FIELD_LASTMODIFIEDDATE     = flockIFavoriteProperties.FIELD_LASTMODIFIEDDATE;
const FIELD_ADDEDDATE            = flockIFavoriteProperties.FIELD_ADDEDDATE;

const FLOCK_WEBSERVICE_DELICIOUS_CONTRACTID = '@flock.com/favorites/webservice;1?name=delicious';
const FLOCK_WEBSERVICE_DELICIOUS_CID        = Components.ID ('{7e8e6aa7-ffdc-484a-a8c3-e95865d91bc0}');
const PREFERENCES_CONTRACTID                = '@mozilla.org/preferences-service;1';
const flockIWebService                      = Components.interfaces.flockIWebService;
const nsIClassInfo                          = Components.interfaces.nsIClassInfo;
const nsIPrefBranch                         = Components.interfaces.nsIPrefBranch;

// prefs
const PREF_LAST_SYNC                        = 'flock.favorites.delicious.lastSync';

// bookmark adding throttling
const THROTTLE_TIME                         = 500;
const THROTTLE_AMOUNT                       = 5;

// services
const PREFS = Components.classes[PREFERENCES_CONTRACTID].getService(nsIPrefBranch);

function DeliciousWebService () {
    this.delicious = new Delicious ('', '');

    // a queue of bookmarks to add
    var localAddQueue = new Array ();
    this.localAddQueue = localAddQueue;

    // a timer to add up to 10 bookmarks every half a second
    this.localAddTimer = Components.classes["@mozilla.org/timer;1"]
        .createInstance(Components.interfaces.nsITimer);
    var callback = new Object ();
    callback.notify = function (timer) {
        if (localAddQueue.length == 0) {
            // if theres nothing to do, do nothing
            return;
        }

        var fsvc = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-favorites'].getService (Components.interfaces.flockIFavoritesService);

        for (var i=0; i<THROTTLE_AMOUNT && localAddQueue.length>0; i++) {
            var fav = localAddQueue.shift ();
            if (fsvc.isFavorite (fav.url)) {
                fsvc.updateFavorite (fav.url, fav);
            } else {
                fsvc.createFavorite (fav.url, fav);
            }
        }
    }
    this.localAddTimer.initWithCallback (callback, THROTTLE_TIME,
            Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
}
DeliciousWebService.prototype._getLastSync = function () {
    try {
        var lastSyncInt = PREFS.getIntPref (PREF_LAST_SYNC);
        var lastSyncDate = new Date ();
        lastSyncDate.setTime (lastSyncInt*1000);
        return lastSyncDate;
    } catch (e) { }
    return null;
}
DeliciousWebService.prototype._setLastSync = function (aDate) {
    try {
        if (aDate == null) {
            dump ('DeliciousWebService._setLastSync got aDate==null - resetting pref\n');
            PREFS.clearUserPref (PREF_LAST_SYNC);
            return;
        }
        var lastSyncInt = aDate.getTime ()/1000;
        PREFS.setIntPref (PREF_LAST_SYNC, lastSyncInt);
    } catch (e) { 
        dump ('exception setting LastSync date: '+e+'\n');
    }
}
// the flockIWebService implementation
DeliciousWebService.prototype.add = function (aURL, aProperties, aListener) { 
    var args = { replace: 'yes', url: aURL };
    if (aProperties.fields & FIELD_NAME) {
        args.description = aProperties.name;
    }
    if (aProperties.fields & FIELD_DESCRIPTION) {
        args.extended = aProperties.description;
    }
    if (aProperties.fields & FIELD_TAGS) {
        /* the translation is something like this: 
             - remove spaces surrounding commas
             - turn spaces into pluses
             - turn commas into spaces
         */
        args.tags = aProperties.tags.
            // remove extra whitespace
            replace (/^ */, '').replace(/ *$/, '').replace (/ *, */g, ',').
            // replace spaces with _ and commas with spaces
            replace (/ /g, '_').replace (/,/g, ' ');
        dump ('USING TAGS: "'+args.tags+'"\n');
    }

    var call = new DeliciousCall ('posts', 'add', args);
    call.delicious = this;
    call.listener = aListener;
    call.onError = function (message) { aListener.onError (message); this.delicious.delicious.submit(this); }
    call.onResult = function (message, xml) { 
        result = this.delicious._parseResult (xml);
        if (result == 'done') {
            aListener.onSuccess (); 
        } else {
            aListener.onError (result);
        }
    }
    call.onTimeout = function () { aListener.onError ('Timeout'); this.delicious.delicious.submit(this); }

    this.delicious.submit (call);
}
DeliciousWebService.prototype._parseResult = function (xml) {
    var results = xml.getElementsByTagName ('result');
    if (results.length != 1) {
        return null;
    }
    var code = results.item(0).attributes.getNamedItem ('code');
    return code.nodeValue;
}
DeliciousWebService.prototype._parsePosts = function (xml) {
    var posts = new Array ();
    var posts_xml = xml.getElementsByTagName ('post');
    for (var i=0; i<posts_xml.length; i++) {
        var post_xml = posts_xml.item (i);
        var post_attrs = post_xml.attributes;
        var post = new Object ();
        for (var j=0; j<post_attrs.length; j++) {
            var attr_node = post_attrs[j];
            if (attr_node.nodeName == 'href') {
                post.url = attr_node.nodeValue;
            } else if (attr_node.nodeName == 'description') {
                post.name = attr_node.nodeValue;
            } else if (attr_node.nodeName == 'extended') {
                post.description = attr_node.nodeValue;
            } else if (attr_node.nodeName == 'tag') {
                if (attr_node.nodeValue == 'system:unfiled') {
                    posts.tags = '';
                } else {
                    post.tags = attr_node.nodeValue;
                    /* translating from delicious back to flock tags:
                        - replace spaces with commas
                        - replace pluses with spaces
                    */
                    post.tags = post.tags
                        .replace (/ /g, ',').replace (/_/g, ' ');
                }
            } else if (attr_node.nodeName == 'time') {
                post.date = parseDate (attr_node.nodeValue);
            }
        }
        posts.push (post);
    }
    return posts;
}
DeliciousWebService.prototype.remove = function (aURL, aListener) { 
    var call = new DeliciousCall ('posts', 'delete', {url: aURL});
    call.delicious = this;
    call.listener = aListener;
    call.onError = function (message) { aListener.onError (message); this.delicious.delicious.submit(this); }
    call.onResult = function (message, xml) { 
        result = this.delicious._parseResult (xml);
        if (result == 'done') {
            aListener.onSuccess (); 
        } else {
            aListener.onError (result);
        }
    }
    call.onTimeout = function () { aListener.onError ('Timeout'); this.delicious.delicious.submit(this); }

    this.delicious.submit (call);
}
DeliciousWebService.prototype.change = function (aURL, aProperties, aListener) { 
    // in delicious, change==add
    this.add (aURL, aProperties, aListener);
}
DeliciousWebService.prototype.authenticate = function (aUsername, aPassword, aListener) { 
    // FIXME: handle authentication failure
    // FIXME: remove authentication from this

    // try to get a non-existent tag
    var call = new DeliciousCall ('posts', 'get', {tag:'flock-imaginary-tag'});
    call.onError = function (message) { 
        aListener.onError (message); 
        this.delicious.delicious.submit(this);  
    }
    call.onResult = function (message, xml) { 
        aListener.onSuccess (); 
    }
    call.onTimeout = function () { 
        aListener.onError ('Timeout'); 
        this.delicious.delicious.submit(this);  
    }

    this.delicious.submit (call);
}
DeliciousWebService.prototype.createCollection = function (aCollectionName, aListener) { aListener.onError ('Not Implemented'); }
DeliciousWebService.prototype.deleteCollection = function (aCollectionName, aListener) { aListener.onError ('Not Implemented'); }
DeliciousWebService.prototype.updateFoler = function (aCollectionName, aListener) { aListener.onError ('Not Implemented'); }
DeliciousWebService.prototype._addPosts = function (xml) {
    var fsvc = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-favorites'].getService (Components.interfaces.flockIFavoritesService);
    var lastSync = this._getLastSync ();
    var posts = this._parsePosts (xml);
    var latest_update = new Date ();
    latest_update.setTime (lastSync?lastSync.getTime():0);
    for (i in posts) {
        var post = posts[i];

        if (!post || !post.url) {
            continue;
            // work out why this happens sometimes
        }

        if (lastSync && lastSync>=post.date) {
            // if we've synced before and the last sync time is more 
            // recent than post, lets ignore it
            continue;
        }

        if (post.date > latest_update) {
            latest_update = post.date;
        }

        var fav = new Object ();
        fav.fields = FIELD_NAME|FIELD_DESCRIPTION|FIELD_TAGS|FIELD_NSYNC
        fav.name = post.name?post.name:'';
        fav.description = post.description?post.description:'';
        fav.tags = post.tags?post.tags:'';
        // mark this favorite as in sync
        fav.nsync = true;

        // queue this favorite for adding asynchronously
        fav.url = post.url;
        this.localAddQueue.push (fav);
    }
    this._setLastSync (latest_update);
}
DeliciousWebService.prototype._getAll = function (aListener) { 
    // get all posts
    var call = new DeliciousCall ('posts', 'all');
    call.delicious = this;
    call.listener = aListener;
    call.onError = function (message) { 
        aListener.onError (message); 
        this.delicious.delicious.submit(this);  
    }
    call.onResult = function (message, xml) { 
        this.delicious._addPosts (xml);

        aListener.onSuccess (); 
    }
    call.onTimeout = function () { aListener.onError ('Timeout'); this.delicious.delicious.submit(this); }

    this.delicious.submit (call);
}
DeliciousWebService.prototype._getRecent = function (aListener) { 
    // get all posts
    var call = new DeliciousCall ('posts', 'recent', {count:50});
    call.delicious = this;
    call.listener = aListener;
    call.onError = function (message) { aListener.onError (message); this.delicious.delicious.submit(this); }
    call.onResult = function (message, xml) { 
        // lets look to make sure we have posts at least as old as lastSync
        var lastSync = this.delicious._getLastSync ();
        var posts = this.delicious._parsePosts (xml);
        for (i=0; i<posts.length; i++) {
            if (posts[i].date <= lastSync) {
                // FIXME: don't parse xml twice
                this.delicious._addPosts (xml);
                aListener.onSuccess ();
                return;
            }
        }
        this.delicious._getAll (aListener);
    }
    call.onTimeout = function () { aListener.onError ('Timeout'); this.delicious.delicious.submit(this); }

    this.delicious.submit (call);
}
DeliciousWebService.prototype.syncDown = function (aListener) { 
    dump ('syncDown ('+aListener+')\n');
    var lastSync = this._getLastSync ();
    dump ('  lastSync: '+lastSync+'\n');

    if (!lastSync) {
        // we've never synced - lets bring down all the posts
        this._getAll (aListener);
        return;
    }

    // get the last post time
    var call = new DeliciousCall ('posts', 'update');
    call.delicious = this;
    call.listener = aListener;
    call.onError = function (message) { aListener.onError (message); this.delicious.delicious.submit(this); }
    call.onResult = function (message, xml) { 
        var updates_xml = xml.getElementsByTagName ('update');
        if (updates_xml == null || updates_xml.length !=1) {
            aListener.onError ('failed to parse update response');
            return;
        }
        var update_xml = updates_xml.item (0);
        var attrs = update_xml.attributes;
        for (var i=0; i<attrs.length; i++) {
            if (attrs[i].nodeName == 'time') {
                var update = parseDate (attrs[i].nodeValue);
                if (update == null) {
                    aListener.onError ('failed to parse update date');
                    return;
                }
                if (update > lastSync) {
                    // there have been changes
                    this.delicious._getRecent (aListener);
                    return;
                }
                // no changes = we're good
                aListener.onSuccess ();
            }
        }
        aListener.onError ('failed to parse update response');
    }
    call.onTimeout = function () { aListener.onError ('Timeout'); }

    this.delicious.submit (call);

    dump ('WEB URL: '+this.getWebView ()+'\n');
}
DeliciousWebService.prototype.getWebView = function () {
    var url = 'http://del.icio.us/';
    var user = null;

    // look for our username in the password manager
    var passman = Components.classes['@mozilla.org/passwordmanager;1'].getService (Components.interfaces.nsIPasswordManager);
    var passwords = passman.enumerator;
    while (passwords.hasMoreElements ()) {
        var pass = passwords.getNext ().QueryInterface (Components.interfaces.nsIPassword);
        if (pass.host != 'del.icio.us:80 (del.icio.us API)') {
            continue;
        }
        return url+pass.user+'/';
    }
    return url;
}

DeliciousWebService.prototype.setEnabled = function (aEnabled) { 
    this.delicious.setEnabled(aEnabled);
}

// the nsIClassInfo implementation
DeliciousWebService.prototype.flags = nsIClassInfo.Singleton;
DeliciousWebService.prototype.classDescription = "Flock Delicious Web Service";
DeliciousWebService.prototype.getInterfaces = function (count) {
    var interfaceList = [flockIWebService, nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
}
DeliciousWebService.prototype.getHelperForLanguage = function (count) { return null; }

// the nsISupports implementation
DeliciousWebService.prototype.QueryInterface =
function (iid) {
    if (!iid.equals(flockIWebService) && 
        !iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
}



// component module glue code
var deliciousModule = new Object();

deliciousModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(FLOCK_WEBSERVICE_DELICIOUS_CID, 
                                    "Flock Delicious JS Component",
                                    FLOCK_WEBSERVICE_DELICIOUS_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
    const CATMAN_CONTRACTID = "@mozilla.org/categorymanager;1";
    const nsICategoryManager = Components.interfaces.nsICategoryManager;
    var catman = Components.classes[CATMAN_CONTRACTID].
                            getService(nsICategoryManager);
                            
    const JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY = "JavaScript global property";
    catman.addCategoryEntry(JAVASCRIPT_GLOBAL_PROPERTY_CATEGORY,
                            "flockWebServiceDelicious",
                            FLOCK_WEBSERVICE_DELICIOUS_CONTRACTID,
                            true,
                            true);
}

deliciousModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(FLOCK_WEBSERVICE_DELICIOUS_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return deliciousServiceFactory;
}

deliciousModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    return true;
}
    
/* factory object */
var deliciousServiceFactory = new Object();

deliciousServiceFactory.createInstance =
function (outer, iid) {
    debug("CI: " + iid);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new DeliciousWebService()).QueryInterface(iid);
}

/* entrypoint */
function NSGetModule(compMgr, fileSpec) {
    return deliciousModule;
}
